"""Test for the concrete structural class
according to Table 4.3 from NEN-EN 1992-1-1:2005+A1:2015+NB:2016+A1:2020: Chapter 4 - Durability and cover to reinforcement.
"""

import pytest

from blueprints.codes.eurocode.nen_en_1992_1_1_a1_2020.chapter_4_durability_and_cover.table_4_1 import (
    Carbonation,
    Chemical,
    Chloride,
    ChlorideSeawater,
    FreezeThaw,
    Table4Dot1ExposureClasses,
)
from blueprints.codes.eurocode.nen_en_1992_1_1_a1_2020.chapter_4_durability_and_cover.table_4_3 import (
    ConcreteMaterial,
    ConcreteStrengthClass,
    ConcreteStructuralClassBase,
    ConcreteStructuralClassCalculator,
    Table4Dot3ConcreteStructuralClass,
)
from blueprints.type_alias import YEARS

DUMMY_EXPOSURE_CLASSES_NA = Table4Dot1ExposureClasses()

DUMMY_EXPOSURE_CLASSES = Table4Dot1ExposureClasses(
    carbonation=Carbonation.XC2,
    chloride=Chloride.XD1,
    chloride_seawater=ChlorideSeawater.XS1,
    freeze_thaw=FreezeThaw.NA,
    chemical=Chemical.XA2,
)


class TestConcreteStructuralClassCalculator:
    """Unit tests for the ConcreteStructuralClassCalculator class."""

    @pytest.mark.parametrize(
        ("design_working_life", "expected_structural_class", "expected_explanation"),
        [
            (40, 4, " + 0 class (40 < 50 => 50 years)"),
            (50, 4, " + 0 class (50 years)"),
            (60, 5, " + 1 class (50 < 60 < 75 => 75 years)"),
            (75, 5, " + 1 class (75 years)"),
            (90, 6, " + 2 classes (90 > 75 => 100 years)"),
            (100, 6, " + 2 classes (100 years)"),
        ],
    )
    def test_structural_class_delta_design_working_life(
        self, design_working_life: YEARS, expected_structural_class: int, expected_explanation: str
    ) -> None:
        """Test the _structural_class_delta_design_working_life method of the ConcreteStructuralClassCalculator class."""
        # Using the default values for the other parameters so that they do not influence the test
        calculator = ConcreteStructuralClassCalculator(
            DUMMY_EXPOSURE_CLASSES_NA,
            design_working_life,
            ConcreteMaterial(ConcreteStrengthClass("C20/25")),
            False,
            False,
        )

        calculator.calculate_structural_class()

        assert calculator.structural_class == expected_structural_class
        assert expected_explanation in calculator.explanation

    @pytest.mark.parametrize(
        ("plate_geometry", "expected_structural_class", "expected_explanation"),
        [(True, 3, " - 1 class (plate geometry)"), (False, 4, " + 0 class (no plate geometry)")],
    )
    def test_structural_class_delta_plate_geometry(self, plate_geometry: bool, expected_structural_class: int, expected_explanation: str) -> None:
        """Test the _structural_class_delta_plate_geometry method of the ConcreteStructuralClassCalculator class."""
        # Using the default values for the other parameters so that they do not influence the test
        calculator = ConcreteStructuralClassCalculator(
            DUMMY_EXPOSURE_CLASSES_NA,
            50,
            ConcreteMaterial(ConcreteStrengthClass("C20/25")),
            plate_geometry,
            False,
        )

        calculator.calculate_structural_class()

        assert calculator.structural_class == expected_structural_class
        assert expected_explanation in calculator.explanation

    @pytest.mark.parametrize(
        ("quality_control", "expected_structural_class", "expected_explanation"),
        [(True, 3, " - 1 class (quality control)"), (False, 4, " + 0 class (no quality control)")],
    )
    def test_structural_class_delta_quality_control(self, quality_control: bool, expected_structural_class: int, expected_explanation: str) -> None:
        """Test the _structural_class_delta_quality_control method of the ConcreteStructuralClassCalculator class."""
        # Using the default values for the other parameters so that they do not influence the test
        calculator = ConcreteStructuralClassCalculator(
            DUMMY_EXPOSURE_CLASSES_NA,
            50,
            ConcreteMaterial(ConcreteStrengthClass("C20/25")),
            False,
            quality_control,
        )

        calculator.calculate_structural_class()

        assert calculator.structural_class == expected_structural_class
        assert expected_explanation in calculator.explanation

    @pytest.mark.parametrize(
        ("concrete_grade", "exposure_classes", "expected_structural_class", "expected_explanation"),
        [
            (
                ConcreteMaterial(ConcreteStrengthClass("C20/25")),
                Table4Dot1ExposureClasses(Carbonation.NA, Chloride.NA, ChlorideSeawater.NA, FreezeThaw.NA, Chemical.NA),
                4,
                " + 0 class (no reduction with respect to concrete grade)",
            ),
            (
                ConcreteMaterial(ConcreteStrengthClass("C30/37")),
                Table4Dot1ExposureClasses(Carbonation.XC1, Chloride.NA, ChlorideSeawater.NA, FreezeThaw.NA, Chemical.NA),
                3,
                " - 1 class (concrete grade >= C30/37)",
            ),
            (
                ConcreteMaterial(ConcreteStrengthClass("C35/45")),
                Table4Dot1ExposureClasses(Carbonation.XC2, Chloride.NA, ChlorideSeawater.NA, FreezeThaw.NA, Chemical.NA),
                3,
                " - 1 class (concrete grade >= C35/45)",
            ),
            (
                ConcreteMaterial(ConcreteStrengthClass("C40/50")),
                Table4Dot1ExposureClasses(Carbonation.NA, Chloride.NA, ChlorideSeawater.XS1, FreezeThaw.NA, Chemical.NA),
                3,
                " - 1 class (concrete grade >= C40/50)",
            ),
            (
                ConcreteMaterial(ConcreteStrengthClass("C45/55")),
                Table4Dot1ExposureClasses(Carbonation.NA, Chloride.XD3, ChlorideSeawater.NA, FreezeThaw.NA, Chemical.NA),
                3,
                " - 1 class (concrete grade >= C45/55)",
            ),
        ],
    )
    def test_structural_class_delta_concrete_grade(
        self, concrete_grade: ConcreteMaterial, exposure_classes: Table4Dot1ExposureClasses, expected_structural_class: int, expected_explanation: str
    ) -> None:
        """Test the _structural_class_delta_concrete_grade method of the ConcreteStructuralClassCalculator class."""
        # Using the default values for the other parameters so that they do not influence the test
        calculator = ConcreteStructuralClassCalculator(
            exposure_classes=exposure_classes,
            concrete_material=concrete_grade,
            design_working_life=50,
            plate_geometry=False,
            quality_control=False,
        )

        calculator.calculate_structural_class()

        assert calculator.structural_class == expected_structural_class
        assert expected_explanation in calculator.explanation


class TestTable4Dot3ConcreteStructuralClass:
    """Unit tests for the Table4Dot3ConcreteStructuralClass class."""

    def test_new_instance(self) -> None:
        """Test if a new instance can be instantiated."""
        new_instance = Table4Dot3ConcreteStructuralClass(
            exposure_classes=DUMMY_EXPOSURE_CLASSES,
            design_working_life=50,
            concrete_material=ConcreteMaterial(ConcreteStrengthClass("C20/25")),
            plate_geometry=False,
            quality_control=False,
        )

        assert isinstance(new_instance, Table4Dot3ConcreteStructuralClass)
        assert isinstance(new_instance, ConcreteStructuralClassBase)
        assert isinstance(new_instance, int)

    def test_new_instance_list_of_exposure_classes(self) -> None:
        """Test if a new instance can be instantiated with a list of exposure classes."""
        new_instance = Table4Dot3ConcreteStructuralClass(
            exposure_classes=["XC2", "XD1", "XS1", "XA2"],
            design_working_life=50,
            concrete_material=ConcreteMaterial(ConcreteStrengthClass("C20/25")),
            plate_geometry=False,
            quality_control=False,
        )

        assert isinstance(new_instance, Table4Dot3ConcreteStructuralClass)
        assert isinstance(new_instance, ConcreteStructuralClassBase)
        assert isinstance(new_instance, int)

    def test_new_instance_attributes(self) -> None:
        """Test if the new instance has the 'explanation' attribute."""
        new_instance = Table4Dot3ConcreteStructuralClass(
            exposure_classes=DUMMY_EXPOSURE_CLASSES,
            design_working_life=50,
            concrete_material=ConcreteMaterial(ConcreteStrengthClass("C20/25")),
            plate_geometry=False,
            quality_control=False,
        )

        assert hasattr(new_instance, "explanation")



"""Formula 4.1 from NEN-EN 1992-1-1:2005+A1:2015+NB:2016+A1:2020: Chapter 4 - Durability and cover to reinforcement."""

from blueprints.codes.eurocode.nen_en_1992_1_1_a1_2020 import NEN_EN_1992_1_1_A1_2020
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula
from blueprints.type_alias import MM
from blueprints.validations import raise_if_negative


class Form4Dot1NominalConcreteCover(Formula):
    r"""Class representing the formula 4.1 for the calculation of the nominal concrete cover [$c_{nom}$] [$mm$]."""

    label = "4.1"
    source_document = NEN_EN_1992_1_1_A1_2020

    def __init__(
        self,
        c_min: MM,
        delta_c_dev: MM,
    ) -> None:
        r"""[$c_{nom}$] Calculates the nominal concrete cover [$mm$].

        Please be advised that this formula does not take various considerations in art.4.4.1.2 and 4.4.1.3 into account.
        For a more detailed calculation, please refer to the NominalConcreteCover class.

        NEN-EN 1992-1-1:2005+A1:2015+NB:2016+A1:2020 art.4.4.1.1 (2) - Formula (4.1)

        Parameters
        ----------
        c_min: MM
            [$c_{min}$] Minimum concrete cover based on art. 4.4.1.2 [$mm$].
        delta_c_dev: MM
            [$\Delta c_{dev}$] Construction tolerance based on art. 4.4.1.3 [$mm$].
        """
        super().__init__()
        self.c_min = c_min
        self.delta_c_dev = delta_c_dev

    @staticmethod
    def _evaluate(
        c_min: MM,
        delta_c_dev: MM,
    ) -> MM:
        """For more detailed documentation see the class docstring."""
        raise_if_negative(c_min=c_min, delta_c_dev=delta_c_dev)
        return c_min + delta_c_dev

    def latex(self, n: int = 3) -> LatexFormula:  # noqa: ARG002
        """Returns LatexFormula object for formula 4.1."""
        return LatexFormula(
            return_symbol=r"c_{nom}",
            result=str(self),
            equation=r"c_{min}+\Delta c_{dev}",
            numeric_equation=f"{self.c_min}+{self.delta_c_dev}",
            comparison_operator_label="=",
        )



"""Testing formula 6.50 of EN 1992-1-1:2004."""

import pytest

from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_6_ultimate_limit_state.formula_6_50 import Form6Dot50PunchingStressResistance
from blueprints.validations import LessOrEqualToZeroError, NegativeValueError


class TestForm6Dot50PunchingStressResistance:
    """Validation for formula 6.50 from EN 1992-1-1:2004."""

    def test_evaluation(self) -> None:
        """Tests the evaluation of the result."""
        # Example values
        c_rd_c = 0.18
        k = 1.5
        rho_l = 0.02
        f_ck = 30.0
        d = 500.0
        a = 300.0
        v_min = 0.035

        # Object to test
        formula = Form6Dot50PunchingStressResistance(c_rd_c=c_rd_c, k=k, rho_l=rho_l, f_ck=f_ck, d=d, a=a, v_min=v_min)

        # Expected result, manually calculated
        manually_calculated_result = 3.52338087705  # MPa

        assert formula == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    @pytest.mark.parametrize(
        ("c_rd_c", "k", "rho_l", "f_ck", "d", "a", "v_min"),
        [
            (-0.18, 1.5, 0.02, 30.0, 500.0, 300.0, 0.035),  # c_rd_c is negative
            (0.18, -1.5, 0.02, 30.0, 500.0, 300.0, 0.035),  # k is negative
            (0.18, 1.5, -0.02, 30.0, 500.0, 300.0, 0.035),  # rho_l is negative
            (0.18, 1.5, 0.02, -30.0, 500.0, 300.0, 0.035),  # f_ck is negative
            (0.18, 1.5, 0.02, 30.0, -500.0, 300.0, 0.035),  # d is negative
            (0.18, 1.5, 0.02, 30.0, 500.0, -300.0, 0.035),  # a is negative
            (0.18, 1.5, 0.02, 30.0, 500.0, 300.0, -0.035),  # v_min is negative
            (0.18, 1.5, 0.02, 30.0, 500.0, 0.0, 0.035),  # a is zero
        ],
    )
    def test_raise_error_when_invalid_values_are_given(
        self, c_rd_c: float, k: float, rho_l: float, f_ck: float, d: float, a: float, v_min: float
    ) -> None:
        """Test invalid values."""
        with pytest.raises((NegativeValueError, LessOrEqualToZeroError)):
            Form6Dot50PunchingStressResistance(c_rd_c=c_rd_c, k=k, rho_l=rho_l, f_ck=f_ck, d=d, a=a, v_min=v_min)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"v_{Rd} = \max \left( C_{Rd,c} \cdot k \cdot \left( 100 \cdot \rho_l \cdot f_{ck} \right)^{\frac{1}{3}} "
                r"\cdot \frac{2 \cdot d}{a}, v_{min} \cdot \frac{2 \cdot d}{a} \right)"
                r" = \max \left( 0.180 \cdot 1.500 \cdot \left( 100 \cdot 0.020 \cdot 30.000 \right)^{\frac{1}{3}} \cdot "
                r"\frac{2 \cdot 500.000}{300.000}, 0.035 \cdot \frac{2 \cdot 500.000}{300.000} \right) = 3.523 \ MPa",
            ),
            (
                "complete_with_units",
                r"v_{Rd} = \max \left( C_{Rd,c} \cdot k \cdot \left( 100 \cdot \rho_l \cdot f_{ck} \right)^{\frac{1}{3}} "
                r"\cdot \frac{2 \cdot d}{a}, v_{min} \cdot \frac{2 \cdot d}{a} \right)"
                r" = \max \left( 0.180 \cdot 1.500 \cdot \left( 100 \cdot 0.020 \cdot 30.000 \ MPa \right)^{\frac{1}{3}} "
                r"\cdot \frac{2 \cdot 500.000 \ mm}{300.000 \ mm}, 0.035 \ MPa \cdot \frac{2 \cdot 500.000 "
                r"\ mm}{300.000 \ mm} \right) = 3.523 \ MPa",
            ),
            ("short", r"v_{Rd} = 3.523 \ MPa"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        c_rd_c = 0.18
        k = 1.5
        rho_l = 0.02
        f_ck = 30.0
        d = 500.0
        a = 300.0
        v_min = 0.035

        # Object to test
        latex = Form6Dot50PunchingStressResistance(c_rd_c=c_rd_c, k=k, rho_l=rho_l, f_ck=f_ck, d=d, a=a, v_min=v_min).latex()

        actual = {
            "complete": latex.complete,
            "short": latex.short,
            "complete_with_units": latex.complete_with_units,
        }

        assert expected == actual[representation], f"{representation} representation failed."



"""Formula 7.11 from EN 1992-1-1:2004: Chapter 7 - Serviceability Limit State."""

from blueprints.codes.eurocode.en_1992_1_1_2004 import EN_1992_1_1_2004
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula, latex_replace_symbols
from blueprints.type_alias import DIMENSIONLESS, MM
from blueprints.validations import raise_if_less_or_equal_to_zero, raise_if_negative


class Form7Dot11MaximumCrackSpacing(Formula):
    r"""Class representing formula 7.11 for the calculation of [$s_{r,max}$]."""

    label = "7.11"
    source_document = EN_1992_1_1_2004

    def __init__(
        self,
        k_3: DIMENSIONLESS,
        c: MM,
        k_1: DIMENSIONLESS,
        k_2: DIMENSIONLESS,
        k_4: DIMENSIONLESS,
        diam: MM,
        rho_p_eff: DIMENSIONLESS,
    ) -> None:
        r"""[$s_{r,max}$] Calculation of the maximum crack spacing [$mm$].

        EN 1992-1-1:2004 art.7.3.4(3) - Formula (7.11)

        Parameters
        ----------
        k_3 : DIMENSIONLESS
            [$k_3$] Coefficient, the recommended value is 3.4 [$-$].
        c : MM
            [$c$] Cover to the longitudinal reinforcement [$mm$].
        k_1 : DIMENSIONLESS
            [$k_1$] Coefficient which takes account of the bond properties of the bonded reinforcement.
            Use 0.8 for high bond bars and 1.6 for bars with an efffectively plain surface
            (e.g. prestressing tendons) [$-$].
        k_2 : DIMENSIONLESS
            [$k_2$] Coefficient which takes account of the distribution of strain.
            Use 0.5 for bending, 1.0 for pure tension. For cases of eccentric tension or for local areas,
            intermediate values of k2 should be used which may be calculated with equation (7.13) [$-$] .
        k_4 : DIMENSIONLESS
            [$k_4$] Coefficient, the recommended value is 0.425 [$-$].
        diam : MM
            [$diam$] Bar diameter [$mm$]. Where a mixture of bar diameters is used in a section, an
            equivalent diameter, $⌀_{eq}$, should be used. For a section with $n_1$ bars of diameter $⌀_1$
            and $n_2$ bars of diameter $⌀_2$, expression (7.12) should be used to calculate $⌀_{eq}$.
        rho_p_eff : DIMENSIONLESS
            [$\rho_{p,eff}$] Effective reinforcement ratio [$-$].
        """
        super().__init__()
        self.k_3 = k_3
        self.c = c
        self.k_1 = k_1
        self.k_2 = k_2
        self.k_4 = k_4
        self.diam = diam
        self.rho_p_eff = rho_p_eff

    @staticmethod
    def _evaluate(
        k_3: DIMENSIONLESS,
        c: MM,
        k_1: DIMENSIONLESS,
        k_2: DIMENSIONLESS,
        k_4: DIMENSIONLESS,
        diam: MM,
        rho_p_eff: DIMENSIONLESS,
    ) -> MM:
        """Evaluates the formula, for more information see the __init__ method."""
        raise_if_negative(k_3=k_3, c=c, k_1=k_1, k_2=k_2, k_4=k_4, diam=diam)
        raise_if_less_or_equal_to_zero(rho_p_eff=rho_p_eff)

        return k_3 * c + k_1 * k_2 * k_4 * diam / rho_p_eff

    def latex(self, n: int = 3) -> LatexFormula:
        """Returns LatexFormula object for formula 7.11."""
        _equation: str = r"k_3 \cdot c + k_1 \cdot k_2 \cdot k_4 \cdot \frac{⌀}{\rho_{p,eff}}"
        _numeric_equation: str = latex_replace_symbols(
            _equation,
            {
                r"k_3": f"{self.k_3:.{n}f}",
                r"c ": f"{self.c:.{n}f} ",
                r"k_1": f"{self.k_1:.{n}f}",
                r"k_2": f"{self.k_2:.{n}f}",
                r"k_4": f"{self.k_4:.{n}f}",
                r"⌀": f"{self.diam:.{n}f}",
                r"\rho_{p,eff}": f"{self.rho_p_eff:.{n}f}",
            },
            False,
        )
        return LatexFormula(
            return_symbol=r"s_{r,max}",
            result=f"{self:.{n}f}",
            equation=_equation,
            numeric_equation=_numeric_equation,
            comparison_operator_label="=",
            unit="mm",
        )



"""Reinforced concrete cross-section example."""

from blueprints.materials.concrete import ConcreteMaterial, ConcreteStrengthClass
from blueprints.materials.reinforcement_steel import ReinforcementSteelMaterial, ReinforcementSteelQuality
from blueprints.structural_sections.concrete.reinforced_concrete_sections.circular import CircularReinforcedCrossSection

# Define a concrete material
concrete = ConcreteMaterial(concrete_class=ConcreteStrengthClass.C35_45)

# Define a reinforcement steel material
steel = ReinforcementSteelMaterial(steel_quality=ReinforcementSteelQuality.B500B)

# Define a circular reinforced cross-section
cs = CircularReinforcedCrossSection(
    diameter=400,
    cover=35,
    concrete_material=concrete,
)

# Add longitudinal reinforcement to the cross-section
cs.add_longitudinal_reinforcement_by_quantity(
    n=5,
    diameter=25,
    material=steel,
)

# Add longitudinal reinforcement to the cross-section
cs.add_longitudinal_reinforcement_by_quantity(
    n=5,
    diameter=16,
    material=steel,
    start_angle=45,
)

# Add stirrups to the cross-section
cs.add_stirrup_along_perimeter(
    diameter=10,
    distance=150,
    material=steel,
)

# Plot the cross-section
cs.plot(show=True)



"""Formula 9.16 from EN 1992-1-1:2004: Chapter 9 - Detailling and specific rules."""

from blueprints.codes.eurocode.en_1992_1_1_2004 import EN_1992_1_1_2004
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula
from blueprints.type_alias import KN, KN_M, M
from blueprints.validations import raise_if_negative


class Form9Dot16MinimumForceOnInternalBeamLine(Formula):
    """Class representing the formula 9.16 for calculating the minimum force on an internal beam line for floors without screeds."""

    label = "9.16"
    source_document = EN_1992_1_1_2004

    def __init__(
        self,
        q_3: KN_M,
        l_1: M,
        l_2: M,
        q_4: KN,
    ) -> None:
        r"""[$F_{tie}$] Minimum force on an internal beam line [$kN$].

        EN 1992-1-1:2004 art.9.10.2.3(4) - Formula (9.16)

        Parameters
        ----------
        q_3: KN_M
            [$q_3$] May be found in national annex, recommended value is 20 [$kN/m$].
        l_1: M
            [$l_1$] span length of floor slabs on either side of the beam, see figure 9.15 [$m$].
        l_2: M
            [$l_2$] span length of floor slabs on either side of the beam, see figure 9.15 [$m$].
        q_4: KN
            [$Q_4$] May be found in national annex, recommended value is 70 [$kN$].
        """
        super().__init__()
        self.q_3 = q_3
        self.l_1 = l_1
        self.l_2 = l_2
        self.q_4 = q_4

    @staticmethod
    def _evaluate(
        q_3: KN_M,
        l_1: M,
        l_2: M,
        q_4: KN,
    ) -> KN:
        """For more detailed documentation see the class docstring."""
        raise_if_negative(q_3=q_3, l_1=l_1, l_2=l_2, q_4=q_4)
        return max(q_3 * (l_1 + l_2) / 2, q_4)

    def latex(self, n: int = 2) -> LatexFormula:
        """Returns LatexFormula object for formula 9.16."""
        return LatexFormula(
            return_symbol=r"F_{tie}",
            result=f"{self:.{n}f}",
            equation=r"min(q_3 \cdot (l_1 + l_2) / 2, Q_4)",
            numeric_equation=rf"min({self.q_3:.{n}f} \cdot ({self.l_1:.{n}f} + {self.l_2:.{n}f}) / 2, {self.q_4:.{n}f})",
            comparison_operator_label="=",
        )



"""Formula 5.44 from EN 1992-1-1:2004: Chapter 5 - Structural Analysis."""

from blueprints.codes.eurocode.en_1992_1_1_2004 import EN_1992_1_1_2004
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula
from blueprints.type_alias import AMOUNT, MM2, MPA, N
from blueprints.validations import raise_if_less_or_equal_to_zero, raise_if_lists_differ_in_length, raise_if_negative


class Form5Dot44PrestressLoss(Formula):
    r"""Class representing formula 5.44 for the calculation of the prestress losses, [$\Delta P_{el}$]."""

    label = "5.44"
    source_document = EN_1992_1_1_2004

    def __init__(
        self,
        a_p: MM2,
        e_p: MPA,
        j: list[AMOUNT],
        delta_sigma_c_t: list[MPA],
        e_cm_t: list[MPA],
    ) -> None:
        r"""[$\Delta P_{el}$] Prestress loss [$N$].

        EN 1992-1-1:2004 art.5.10.5.1(2) - Formula (5.44)

        Parameters
        ----------
        a_p : MM2
            [$A_{p}$] Cross-sectional area of the tendon [$mm^2$].
        e_p : MPA
            [$E_{p}$] Modulus of elasticity of the tendon [$MPa$].
        j : list[AMOUNT]
            [$j$] (n-1)/2n, with n the number of identical tendons successively prestressed [$list[-]$].
        delta_sigma_c_t : list[MPA]
            [$\Delta \sigma_{c}(t)$] variation of stress at the centre of gravity of the tendons applied at time t [$list[MPa]$].
        e_cm_t: list[MPA]
            [$E_{cm}(t)$] 0.1% proof stress of prestressing steel [$list[MPa]$].
        """
        super().__init__()
        self.a_p = a_p
        self.e_p = e_p
        self.j = j
        self.delta_sigma_c_t = delta_sigma_c_t
        self.e_cm_t = e_cm_t

    @staticmethod
    def _evaluate(
        a_p: MPA,
        e_p: MPA,
        j: list[AMOUNT],
        delta_sigma_c_t: list[MPA],
        e_cm_t: list[MPA],
    ) -> N:
        """Evaluates the formula, for more information see the __init__ method."""
        raise_if_negative(a_p=a_p, e_p=e_p)

        raise_if_lists_differ_in_length(j=j, delta_sigma_c_t=delta_sigma_c_t, e_cm_t=e_cm_t)

        for x, y, z in zip(j, delta_sigma_c_t, e_cm_t):
            raise_if_negative(x=x, y=y)
            raise_if_less_or_equal_to_zero(z=z)

        return a_p * e_p * sum(x * (y / z) for x, y, z in zip(j, delta_sigma_c_t, e_cm_t))

    def latex(self, n: int = 3) -> LatexFormula:
        """Returns LatexFormula object for formula 5.43."""
        numeric_equation = rf"{self.a_p:.{n}f} \cdot {self.e_p:.{n}f} \cdot \left( "
        for i in range(len(self.j)):
            numeric_equation += rf"\frac{{{self.j[i]} \cdot {self.delta_sigma_c_t[i]:.{n}f}}}{{{self.e_cm_t[i]:.{n}f}}}"
            if i < len(self.j) - 1:
                numeric_equation += " + "
        numeric_equation += r" \right)"

        return LatexFormula(
            return_symbol=r"\Delta P_{el}",
            result=f"{self:.{n}f}",
            equation=r"A_{p} \cdot E_{p} \cdot \sum_{i=1}^{n} \frac{j_{i} \cdot \Delta \sigma_{c,i}(t)}{E_{cm,i}(t)}",
            numeric_equation=numeric_equation,
            comparison_operator_label="=",
            unit="N",
        )



"""Formula 6.39 from EN 1993-1-1:2005: Chapter 6 - Ultimate Limit State."""

from blueprints.codes.eurocode.en_1993_1_1_2005 import EN_1993_1_1_2005
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula, latex_replace_symbols
from blueprints.type_alias import DIMENSIONLESS, NMM
from blueprints.validations import raise_if_less_or_equal_to_zero, raise_if_negative


class Form6Dot39ReducedBendingMomentResistance(Formula):
    r"""Class representing formula 6.39 for the calculation of [$M_{N,y,Rd}$]."""

    label = "6.39"
    source_document = EN_1993_1_1_2005

    def __init__(
        self,
        mpl_y_rd: NMM,
        n: DIMENSIONLESS,
        a_w: DIMENSIONLESS,
    ) -> None:
        r"""[$M_{N,y,Rd}$] Calculation of the reduced bending moment [$Nmm$].

        EN 1993-1-1:2005 art.6.2.9.1(5) - Formula (6.39)

        Parameters
        ----------
        mpl_y_rd : NMM
            [$M_{pl,y,Rd}$] Plastic bending moment resistance about the y-axis [$Nmm$].
        n : DIMENSIONLESS
            [$n$] Axial force ratio, see equation 6.38n (dimensionless).
        a_w : DIMENSIONLESS
            [$a_w$] Reduction factor for the web (dimensionless), see equation 6.39aw.
        """
        super().__init__()
        self.mpl_y_rd = mpl_y_rd
        self.n = n
        self.a_w = a_w

    @staticmethod
    def _evaluate(
        mpl_y_rd: NMM,
        n: DIMENSIONLESS,
        a_w: DIMENSIONLESS,
    ) -> NMM:
        """Evaluates the formula, for more information see the __init__ method."""
        raise_if_negative(mpl_y_rd=mpl_y_rd, n=n, a_w=a_w)
        raise_if_less_or_equal_to_zero(denominator=(1 - 0.5 * a_w))

        return min(mpl_y_rd * (1 - n) / (1 - 0.5 * a_w), mpl_y_rd)

    def latex(self, n: int = 3) -> LatexFormula:
        """Returns LatexFormula object for formula 6.39."""
        _equation: str = r"\min \left( M_{pl,y,Rd} \cdot \frac{1 - n}{1 - 0.5 \cdot a_w}, M_{pl,y,Rd} \right)"
        _numeric_equation: str = latex_replace_symbols(
            _equation,
            {
                r"M_{pl,y,Rd}": f"{self.mpl_y_rd:.{n}f}",
                r" n": f" {self.n:.{n}f}",
                r"a_w": f"{self.a_w:.{n}f}",
            },
            False,
        )
        _numeric_equation_with_units: str = latex_replace_symbols(
            _equation,
            {
                r"M_{pl,y,Rd}": rf"{self.mpl_y_rd:.{n}f} \ Nmm",
                r" n": rf" {self.n:.{n}f}",
                r"a_w": rf"{self.a_w:.{n}f}",
            },
            False,
        )
        return LatexFormula(
            return_symbol=r"M_{N,y,Rd}",
            result=f"{self:.{n}f}",
            equation=_equation,
            numeric_equation=_numeric_equation,
            numeric_equation_with_units=_numeric_equation_with_units,
            comparison_operator_label=r"=",
            unit="Nmm",
        )



"""Testing formula 6.71 of EN 1992-1-1:2004."""

import pytest

from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_6_ultimate_limit_state.formula_6_71 import (
    Form6Dot71CriteriaBasedOnStressRange,
    Form6Dot71CriteriaBasedOnStressRangeLHS,
    Form6Dot71CriteriaBasedOnStressRangeRHS,
)
from blueprints.validations import LessOrEqualToZeroError, NegativeValueError


class TestForm6Dot71CriteriaBasedOnStressRangeLHS:
    """Validation for LHS formula 6.71 from EN 1992-1-1:2004."""

    def test_evaluation(self) -> None:
        """Test the evaluation of the result."""
        # Example values
        gamma_f_fat = 1.5  # -
        delta_sigma_s_equ_n_star = 10.0  # MPa
        form = Form6Dot71CriteriaBasedOnStressRangeLHS(gamma_f_fat=gamma_f_fat, delta_sigma_s_equ_n_star=delta_sigma_s_equ_n_star)

        # Expected result, manually calculated
        expected = 15.0

        assert form == expected

    @pytest.mark.parametrize(
        ("gamma_f_fat", "delta_sigma_s_equ_n_star"),
        [
            (-1, 10),
            (10, -1),
        ],
    )
    def test_raise_error_when_negative_is_given(self, gamma_f_fat: float, delta_sigma_s_equ_n_star: float) -> None:
        """Test a negative value for v_rd_s."""
        with pytest.raises(NegativeValueError):
            Form6Dot71CriteriaBasedOnStressRangeLHS(gamma_f_fat=gamma_f_fat, delta_sigma_s_equ_n_star=delta_sigma_s_equ_n_star)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"\Delta \sigma_{Ed} = \gamma_{F,fat} \cdot \Delta \sigma_{s,equ} (N^*) = 1.500 \cdot 10.000 = 15.000",
            ),
            ("short", r"\Delta \sigma_{Ed} = 15.000"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        gamma_f_fat = 1.5  # -
        delta_sigma_s_equ_n_star = 10.0  # MPa

        # Object to test
        form_latex = Form6Dot71CriteriaBasedOnStressRangeLHS(gamma_f_fat=gamma_f_fat, delta_sigma_s_equ_n_star=delta_sigma_s_equ_n_star).latex()

        actual = {
            "complete": form_latex.complete,
            "short": form_latex.short,
        }

        assert actual[representation] == expected, f"{representation} representation failed."


class TestForm6Dot71CriteriaBasedOnStressRangeRHS:
    """Validation for RHS formula 6.71 from EN 1992-1-1:2004."""

    def test_evaluation(self) -> None:
        """Test the evaluation of the result."""
        # Example values
        gamma_s_fat = 2.0  # -
        delta_sigma_rsk_n_star = 10.0  # MPa
        form = Form6Dot71CriteriaBasedOnStressRangeRHS(gamma_s_fat=gamma_s_fat, delta_sigma_rsk_n_star=delta_sigma_rsk_n_star)

        # Expected result, manually calculated
        expected = 5.0

        assert form == expected

    @pytest.mark.parametrize(
        ("gamma_s_fat", "delta_sigma_rsk_n_star"),
        [
            (-1, 10),
            (0, 10),
        ],
    )
    def test_raise_error_when_gamma_is_not_positive(self, gamma_s_fat: float, delta_sigma_rsk_n_star: float) -> None:
        """Test a non postive values for gamma_s_fat."""
        with pytest.raises(LessOrEqualToZeroError):
            Form6Dot71CriteriaBasedOnStressRangeRHS(gamma_s_fat=gamma_s_fat, delta_sigma_rsk_n_star=delta_sigma_rsk_n_star)

    def test_raise_error_when_delta_sigma_is_negative(self) -> None:
        """Test a negative value for delta_sigma_rsk_n_star."""
        # Example values
        gamma_s_fat = 1.0  # -
        delta_sigma_rsk_n_star = -3  # MPa

        with pytest.raises(NegativeValueError):
            Form6Dot71CriteriaBasedOnStressRangeRHS(gamma_s_fat=gamma_s_fat, delta_sigma_rsk_n_star=delta_sigma_rsk_n_star)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"\Delta \sigma_{Rd} = \frac{\Delta \sigma_{Rsk} (N^*)}{\gamma_{s,fat}} = \frac{10.000}{2.000} = 5.000",
            ),
            ("short", r"\Delta \sigma_{Rd} = 5.000"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        gamma_s_fat = 2.0  # -
        delta_sigma_rsk_n_star = 10.0  # MPa

        # Object to test
        form_latex = Form6Dot71CriteriaBasedOnStressRangeRHS(gamma_s_fat=gamma_s_fat, delta_sigma_rsk_n_star=delta_sigma_rsk_n_star).latex()

        actual = {
            "complete": form_latex.complete,
            "short": form_latex.short,
        }

        assert actual[representation] == expected, f"{representation} representation failed."


class TestForm6Dot71CriteriaBasedOnStressRange:
    """Validation for formula 6.71 from EN 1992-1-1:2004."""

    @pytest.mark.parametrize(
        ("gamma_f_fat", "delta_sigma_s_equ_n_star", "gamma_s_fat", "delta_sigma_rsk_n_star", "result_manual"),
        [
            (1.5, 10.0, 2.0, 40.0, True),
            (1.5, 10.0, 2.0, 30.0, True),
            (1.5, 10.0, 2.0, 20.0, False),
        ],
    )
    def test_evaluation(
        self, gamma_f_fat: float, delta_sigma_s_equ_n_star: float, gamma_s_fat: float, delta_sigma_rsk_n_star: float, result_manual: bool
    ) -> None:
        """Test the evaluation of the result."""
        form = Form6Dot71CriteriaBasedOnStressRange(
            gamma_f_fat=gamma_f_fat,
            delta_sigma_s_equ_n_star=delta_sigma_s_equ_n_star,
            gamma_s_fat=gamma_s_fat,
            delta_sigma_rsk_n_star=delta_sigma_rsk_n_star,
        )

        # Expected result, manually calculated
        expected = result_manual

        assert bool(form) == expected

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                (
                    r"CHECK \rightarrow \gamma_{F,fat} \cdot \Delta \sigma_{s,equ} (N^*) "
                    r"\leq \frac{\Delta \sigma_{Rsk} (N^*)}{\gamma_{s,fat}} \rightarrow "
                    r"1.500 \cdot 10.000 \leq \frac{10.000}{2.000} \rightarrow \text{Not OK}"
                ),
            ),
            ("short", r"CHECK \rightarrow \text{Not OK}"),
        ],
    )
    def test_latex_not_ok(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        gamma_f_fat = 1.5  # -
        delta_sigma_s_equ_n_star = 10.0  # MPa
        gamma_s_fat = 2.0  # -
        delta_sigma_rsk_n_star = 10.0  # MPa

        # Object to test
        form_latex = Form6Dot71CriteriaBasedOnStressRange(
            gamma_f_fat=gamma_f_fat,
            delta_sigma_s_equ_n_star=delta_sigma_s_equ_n_star,
            gamma_s_fat=gamma_s_fat,
            delta_sigma_rsk_n_star=delta_sigma_rsk_n_star,
        ).latex()

        actual = {
            "complete": form_latex.complete,
            "short": form_latex.short,
        }

        assert actual[representation] == expected, f"{representation} representation failed."

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                (
                    r"CHECK \rightarrow \gamma_{F,fat} \cdot \Delta \sigma_{s,equ} (N^*) "
                    r"\leq \frac{\Delta \sigma_{Rsk} (N^*)}{\gamma_{s,fat}} \rightarrow "
                    r"1.500 \cdot 10.000 \leq \frac{40.000}{2.000} \rightarrow OK"
                ),
            ),
            ("short", r"CHECK \rightarrow OK"),
        ],
    )
    def test_latex_ok(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        gamma_f_fat = 1.5  # -
        delta_sigma_s_equ_n_star = 10.0  # MPa
        gamma_s_fat = 2.0  # -
        delta_sigma_rsk_n_star = 40.0  # MPa

        # Object to test
        form_latex = Form6Dot71CriteriaBasedOnStressRange(
            gamma_f_fat=gamma_f_fat,
            delta_sigma_s_equ_n_star=delta_sigma_s_equ_n_star,
            gamma_s_fat=gamma_s_fat,
            delta_sigma_rsk_n_star=delta_sigma_rsk_n_star,
        ).latex()

        actual = {
            "complete": form_latex.complete,
            "short": form_latex.short,
        }

        assert actual[representation] == expected, f"{representation} representation failed."

    def test_property_lhs(self) -> None:
        """Test the latex representation of the formula."""
        # Example values
        gamma_f_fat = 1.5  # -
        delta_sigma_s_equ_n_star = 10.0  # MPa
        gamma_s_fat = 2.0  # -
        delta_sigma_rsk_n_star = 40.0  # MPa

        # Object to test
        form = Form6Dot71CriteriaBasedOnStressRange(
            gamma_f_fat=gamma_f_fat,
            delta_sigma_s_equ_n_star=delta_sigma_s_equ_n_star,
            gamma_s_fat=gamma_s_fat,
            delta_sigma_rsk_n_star=delta_sigma_rsk_n_star,
        ).left_hand_side

        # Expected result, manually calculated
        expected = 15.0

        assert form == expected

    def test_property_rhs(self) -> None:
        """Test the latex representation of the formula."""
        # Example values
        gamma_f_fat = 1.5  # -
        delta_sigma_s_equ_n_star = 10.0  # MPa
        gamma_s_fat = 2.0  # -
        delta_sigma_rsk_n_star = 40.0  # MPa

        # Object to test
        form = Form6Dot71CriteriaBasedOnStressRange(
            gamma_f_fat=gamma_f_fat,
            delta_sigma_s_equ_n_star=delta_sigma_s_equ_n_star,
            gamma_s_fat=gamma_s_fat,
            delta_sigma_rsk_n_star=delta_sigma_rsk_n_star,
        ).right_hand_side

        # Expected result, manually calculated
        expected = 20.0

        assert form == expected

    def test_property_ratio(self) -> None:
        """Test the latex representation of the formula."""
        # Example values
        gamma_f_fat = 1.5  # -
        delta_sigma_s_equ_n_star = 10.0  # MPa
        gamma_s_fat = 2.0  # -
        delta_sigma_rsk_n_star = 40.0  # MPa

        # Object to test
        form = Form6Dot71CriteriaBasedOnStressRange(
            gamma_f_fat=gamma_f_fat,
            delta_sigma_s_equ_n_star=delta_sigma_s_equ_n_star,
            gamma_s_fat=gamma_s_fat,
            delta_sigma_rsk_n_star=delta_sigma_rsk_n_star,
        ).ratio

        # Expected result, manually calculated
        expected = 0.75

        assert form == expected

    def test__str__(self) -> None:
        """Test the string representation of the formula."""
        # Example values
        gamma_f_fat = 1.5
        delta_sigma_s_equ_n_star = 10.0
        gamma_s_fat = 2.0
        delta_sigma_rsk_n_star = 40.0

        # Object to test
        form = Form6Dot71CriteriaBasedOnStressRange(
            gamma_f_fat=gamma_f_fat,
            delta_sigma_s_equ_n_star=delta_sigma_s_equ_n_star,
            gamma_s_fat=gamma_s_fat,
            delta_sigma_rsk_n_star=delta_sigma_rsk_n_star,
        )

        assert isinstance(str(form), str)



"""Formula 6.18 from EN 1992-1-1:2004: Chapter 6 - Ultimate limit state."""

from blueprints.codes.eurocode.en_1992_1_1_2004 import EN_1992_1_1_2004
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula
from blueprints.math_helpers import cot
from blueprints.type_alias import DEG, KN
from blueprints.validations import raise_if_negative


class Form6Dot18AdditionalTensileForce(Formula):
    r"""Class representing formula 6.18 for the calculation of the additional tensile force, [$\Delta F_{td}$]."""

    label = "6.18"
    source_document = EN_1992_1_1_2004

    def __init__(
        self,
        v_ed: KN,
        theta: DEG,
        alpha: DEG,
    ) -> None:
        r"""[$\Delta F_{td}$] Additional tensile force [$kN$].

        EN 1992-1-1:2004 art.6.2.3(7) - Formula (6.18)

        Parameters
        ----------
        v_ed : KN
            [$V_{Ed}$] Design value of the shear force [$kN$].
        theta : DEG
            [$\theta$] Angle between the concrete compression strut and the beam axis perpendicular to the shear force [$degrees$].
        alpha : DEG
            [$\alpha$] Angle between shear reinforcement and the beam axis perpendicular to the shear force [$degrees$].
        """
        super().__init__()
        self.v_ed = v_ed
        self.theta = theta
        self.alpha = alpha

    @staticmethod
    def _evaluate(
        v_ed: KN,
        theta: DEG,
        alpha: DEG,
    ) -> KN:
        """Evaluates the formula, for more information see the __init__ method."""
        raise_if_negative(
            v_ed=v_ed,
            theta=theta,
            alpha=alpha,
        )

        return 0.5 * v_ed * (cot(theta) - cot(alpha))

    def latex(self, n: int = 3) -> LatexFormula:
        """Returns LatexFormula object for formula 6.18."""
        return LatexFormula(
            return_symbol=r"\Delta F_{td}",
            result=f"{self:.{n}f}",
            equation=r"0.5 \cdot V_{Ed} \cdot \left(\cot(\theta) - \cot(\alpha)\right)",
            numeric_equation=rf"0.5 \cdot {self.v_ed:.{n}f} \cdot \left(\cot({self.theta:.{n}f}) - \cot({self.alpha:.{n}f})\right)",
            comparison_operator_label="=",
            unit="kN",
        )



"""Formula 5.48 from EN 1992-1-1:2004: Chapter 5 - Structural Analysis."""

from blueprints.codes.eurocode.en_1992_1_1_2004 import EN_1992_1_1_2004
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula
from blueprints.type_alias import DIMENSIONLESS, KN
from blueprints.validations import raise_if_negative


class Form5Dot48LowerCharacteristicPrestressingValue(Formula):
    r"""Class representing formula 5.48 for the calculation of the lower characteristic value for the prestressing
    value at SLS and Fatigue, [$P_{k,inf}$].
    """

    label = "5.48"
    source_document = EN_1992_1_1_2004

    def __init__(
        self,
        r_inf: DIMENSIONLESS,
        p_m_t: KN,
    ) -> None:
        r"""[$P_{k,inf}$] Lower characteristic value for the prestressing value at SLS and Fatigue [$kN$].

        EN 1992-1-1:2004 art.5.10.9(1) - Formula (5.48)

        Parameters
        ----------
        r_inf : DIMENSIONLESS
            [$r_{inf}$] Factor for the lower characteristic value, recommended value is 0.95 for pre-tensioning or unbounded tendons,
             0.90 for post-tensioning with bonded tendons. When appropriate measures are taken: 1.0 [$-$].
        p_m_t : KN
            [$P_{m,t}(x)$] Mean value of the prestressing force at location x [$kN$].
        """
        super().__init__()
        self.r_inf = r_inf
        self.p_m_t = p_m_t

    @staticmethod
    def _evaluate(
        r_inf: DIMENSIONLESS,
        p_m_t: KN,
    ) -> KN:
        """Evaluates the formula, for more information see the __init__ method."""
        raise_if_negative(
            r_inf=r_inf,
            p_m_t=p_m_t,
        )

        return r_inf * p_m_t

    def latex(self, n: int = 3) -> LatexFormula:
        """Returns LatexFormula object for formula 5.48."""
        return LatexFormula(
            return_symbol=r"P_{k,inf}",
            result=f"{self:.{n}f}",
            equation=r"r_{inf} \cdot P_{m,t}(x)",
            numeric_equation=rf"{self.r_inf:.{n}f} \cdot {self.p_m_t:.{n}f}",
            comparison_operator_label="=",
            unit="kN",
        )



"""Tests for the latex_replace_symbols function."""

import pytest

from blueprints.codes.latex_formula import latex_replace_symbols


# Success tests using pytest
@pytest.mark.parametrize(
    ("latex_template", "replacements", "unique_check", "expected"),
    [
        (r"\frac{A}{B}", {"A": "x", "B": "y"}, True, r"\frac{x}{y}"),
        (r"\frac{A}{B}", {"A": "x", "B": "y"}, False, r"\frac{x}{y}"),
        (r"\frac{A}{B}", {"A": "1", "B": "2"}, False, r"\frac{1}{2}"),
        (
            r"\frac{k_{mod}}{\gamma_{R}}",
            {"k_{mod}": "0.9", r"\gamma_{R}": "1.2"},
            False,
            r"\frac{0.9}{1.2}",
        ),
    ],
)
def test_latex_replace_symbols_success(latex_template: str, replacements: dict[str, str], unique_check: bool, expected: str) -> None:
    """Test successful symbol replacements in LaTeX strings."""
    result = latex_replace_symbols(latex_template, replacements, unique_check)
    assert result == expected


# Error tests using pytest
@pytest.mark.parametrize(
    ("latex_template", "replacements", "unique_check", "expected_exception"),
    [
        (r"\frac{A}{B}", {"C": "x"}, True, "Symbol 'C' not found in the template."),
        (r"\frac{A}{A}", {"A": "x"}, True, "Symbol 'A' found multiple times in the template."),
    ],
)
def test_latex_replace_symbols_error(latex_template: str, replacements: dict[str, str], unique_check: bool, expected_exception: str) -> None:
    """Test error conditions when replacing symbols in LaTeX strings."""
    with pytest.raises(ValueError, match=expected_exception):
        latex_replace_symbols(latex_template, replacements, unique_check)



"""Formula 6.36 from EN 1992-1-1:2004: Chapter 6 - Ultimate limit state."""

from blueprints.codes.eurocode.en_1992_1_1_2004 import EN_1992_1_1_2004
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula
from blueprints.type_alias import MM
from blueprints.validations import raise_if_negative


class Form6Dot36ExternalContourRadiusCircularColumnHeads(Formula):
    r"""Class representing formula 6.36 for the calculation of the contour radius for circular column heads, [$r_{cont,ext}$]."""

    label = "6.36"
    source_document = EN_1992_1_1_2004

    def __init__(
        self,
        l_h: MM,
        d: MM,
        c: MM,
    ) -> None:
        r"""[$r_{cont,ext}$] Contour radius for circular column heads [$mm$].

        EN 1992-1-1:2004 art.6.4.2(11) - Formula (6.36)

        Parameters
        ----------
        l_h : MM
            [$l_{H}$] Distance from the column face to the edge of the column head [$mm$].
        d : MM
            [$d$] Effective depth [$mm$].
        c : MM
            [$c$] Diameter of a circular column [$mm$].
        """
        super().__init__()
        self.l_h = l_h
        self.d = d
        self.c = c

    @staticmethod
    def _evaluate(
        l_h: MM,
        d: MM,
        c: MM,
    ) -> MM:
        """Evaluates the formula, for more information see the __init__ method."""
        raise_if_negative(l_h=l_h, d=d, c=c)
        return l_h + 2 * d + 0.5 * c

    def latex(self, n: int = 3) -> LatexFormula:
        """Returns LatexFormula object for formula 6.36."""
        return LatexFormula(
            return_symbol=r"r_{cont,ext}",
            result=f"{self:.{n}f}",
            equation=r"l_{H} + 2 \cdot d + 0.5 \cdot c",
            numeric_equation=rf"{self.l_h:.{n}f} + 2 \cdot {self.d:.{n}f} + 0.5 \cdot {self.c:.{n}f}",
            comparison_operator_label="=",
            unit="mm",
        )



"""Testing formula 8.15 from EN 1992-1-1:2004."""

import pytest

from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_8_detailing_of_reinforcement_and_prestressing_tendons.formula_8_2 import (
    SubForm8Dot2CoefficientQualityOfBond,
)
from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_8_detailing_of_reinforcement_and_prestressing_tendons.formula_8_15 import (
    Form8Dot15PrestressTransferStress,
    SubForm8Dot15EtaP1,
    SubForm8Dot15TensileStrengthAtRelease,
)
from blueprints.validations import NegativeValueError


class TestForm8Dot15Form8Dot15PrestressTransferStress:
    """Validation for formula 8.15 from EN 1992-1-1:2004."""

    def test_evaluation(self) -> None:
        """Test the evaluation of the result."""
        # example values
        eta_p1 = 2.7  # [-]
        eta_1 = 1  # [-]
        f_ctd_t = 2.5  # MPa
        form_8_15 = Form8Dot15PrestressTransferStress(
            eta_p1=eta_p1,
            eta_1=eta_1,
            f_ctd_t=f_ctd_t,
        )

        # manually calculated result
        manually_calculated_result = 6.75  # MPa

        assert form_8_15 == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    def test_negative_eta_p1(self) -> None:
        """Test the evaluation of the result."""
        # example values
        eta_p1 = -2.7  # [-]
        eta_1 = 1  # [-]
        f_ctd_t = 2.5  # MPa
        with pytest.raises(NegativeValueError):
            Form8Dot15PrestressTransferStress(
                eta_p1=eta_p1,
                eta_1=eta_1,
                f_ctd_t=f_ctd_t,
            )

    def test_negative_eta_1(self) -> None:
        """Test the evaluation of the result."""
        # example values
        eta_p1 = 2.7  # [-]
        eta_1 = -1  # [-]
        f_ctd_t = 2.5  # MPa
        with pytest.raises(NegativeValueError):
            Form8Dot15PrestressTransferStress(
                eta_p1=eta_p1,
                eta_1=eta_1,
                f_ctd_t=f_ctd_t,
            )

    def test_negative_f_ctd_t(self) -> None:
        """Test the evaluation of the result."""
        # example values
        eta_p1 = 2.7  # [-]
        eta_1 = 1  # [-]
        f_ctd_t = -2.5  # MPa
        with pytest.raises(NegativeValueError):
            Form8Dot15PrestressTransferStress(
                eta_p1=eta_p1,
                eta_1=eta_1,
                f_ctd_t=f_ctd_t,
            )

    def test_integration_with_sub_form_8_15_1(self) -> None:
        """Test the integration with sub-formula 8.15."""
        # example values
        eta_1 = 1  # [-]
        f_ctd_t = 2.5  # MPa
        type_of_wire = "indented"
        eta_p1 = SubForm8Dot15EtaP1(type_of_wire=type_of_wire)  # [-]
        form_8_15 = Form8Dot15PrestressTransferStress(eta_p1=eta_p1, eta_1=eta_1, f_ctd_t=f_ctd_t)

        manually_calculated_result = 6.75  # MPa

        assert form_8_15 == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    def test_integration_with_sub_form_8_15_2(self) -> None:
        """Test the integration with sub-formula 8.15."""
        # example values
        eta_p1 = 1  # [-]
        eta_1 = 1  # [-]
        alpha_ct = 1  # [-]
        f_ctm_t = 2.5  # MPa
        gamma_c = 1.5  # [-]
        f_ctd_t = SubForm8Dot15TensileStrengthAtRelease(
            alpha_ct=alpha_ct,
            f_ctm_t=f_ctm_t,
            gamma_c=gamma_c,
        )

        form_8_15 = Form8Dot15PrestressTransferStress(eta_p1=eta_p1, eta_1=eta_1, f_ctd_t=f_ctd_t)

        manually_calculated_result = 1.16666666666666666666  # MPa

        assert form_8_15 == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    def test_integration_with_sub_form_8_2_coefficient_of_bond(self) -> None:
        """Test the integration with sub-formula 8.2."""
        # example values
        eta_p1 = 1
        eta_1 = SubForm8Dot2CoefficientQualityOfBond(bond_quality="good")
        f_ctd_t = 2.5  # MPa

        form_8_15 = Form8Dot15PrestressTransferStress(eta_p1=eta_p1, eta_1=eta_1, f_ctd_t=f_ctd_t)

        manually_calculated_result = 2.5  # MPa

        assert form_8_15 == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                (r"f_{bpt} = \eta_{p1} \cdot \eta_1 \cdot f_{ctd}(t) = 2.70 \cdot 1.00 \cdot 2.50 = 6.75"),
            ),
            ("short", r"f_{bpt} = 6.75"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # example values
        eta_1 = 1  # [-]
        f_ctd_t = 2.5  # MPa
        type_of_wire = "indented"
        eta_p1 = SubForm8Dot15EtaP1(type_of_wire=type_of_wire)  # [-]

        # Object to test
        form_8_15_latex = Form8Dot15PrestressTransferStress(eta_p1=eta_p1, eta_1=eta_1, f_ctd_t=f_ctd_t).latex()

        actual = {"complete": form_8_15_latex.complete, "short": form_8_15_latex.short}

        assert actual[representation] == expected, f"{representation} representation failed."



"""Testing formula 6.27 of EN 1992-1-1:2004."""

import pytest

from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_6_ultimate_limit_state.formula_6_27 import Form6Dot27ShearForceInWall
from blueprints.validations import LessOrEqualToZeroError, NegativeValueError


class TestForm6Dot27ShearForceInWall:
    """Validation for formula 6.27 from EN 1992-1-1:2004."""

    def test_evaluation(self) -> None:
        """Tests the evaluation of the result."""
        # Example values
        tau_t_i_t_ef_i = 100.0
        z_i = 2.0

        # Object to test
        formula = Form6Dot27ShearForceInWall(tau_t_i_t_ef_i=tau_t_i_t_ef_i, z_i=z_i)

        # Expected result, manually calculated
        manually_calculated_result = 200.0

        assert formula == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    @pytest.mark.parametrize(
        ("tau_t_i_t_ef_i", "z_i"),
        [
            (-100.0, 2.0),  # tau_t_i_t_ef_i is negative
            (100.0, -2.0),  # z_i is negative
        ],
    )
    def test_raise_error_when_invalid_values_are_given(self, tau_t_i_t_ef_i: float, z_i: float) -> None:
        """Test invalid values."""
        with pytest.raises((NegativeValueError, LessOrEqualToZeroError)):
            Form6Dot27ShearForceInWall(tau_t_i_t_ef_i=tau_t_i_t_ef_i, z_i=z_i)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"V_{Ed,i} = \tau_{t,i} t_{ef,i} \cdot z_{i} = 100.000 \cdot 2.000 = 200.000 \ N",
            ),
            ("short", r"V_{Ed,i} = 200.000 \ N"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        tau_t_i_t_ef_i = 100.0
        z_i = 2.0

        # Object to test
        latex = Form6Dot27ShearForceInWall(tau_t_i_t_ef_i=tau_t_i_t_ef_i, z_i=z_i).latex()

        actual = {
            "complete": latex.complete,
            "short": latex.short,
        }

        assert expected == actual[representation], f"{representation} representation failed."



"""Formula 5.12 from EN 1993-5:2007: Chapter 5 - Ultimate Limit States."""

import numpy as np

from blueprints.codes.eurocode.en_1993_5_2007 import EN_1993_5_2007
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula
from blueprints.type_alias import DIMENSIONLESS, MM, MM4, MPA, N
from blueprints.validations import raise_if_less_or_equal_to_zero, raise_if_negative


class Form5Dot12ElasticCriticalLoad(Formula):
    r"""Class representing formula 5.12 for the calculation of the elastic critical load, [$N_{cr}$]."""

    label = "5.12"
    source_document = EN_1993_5_2007

    def __init__(
        self,
        e: MPA,
        i: MM4,
        beta_d: DIMENSIONLESS,
        l: MM,  # noqa: E741
    ) -> None:
        r"""[$N_{cr}$] Elastic critical load [$N$].

        EN 1993-5:2007 art.5.2.3 - Formula (5.12)

        Parameters
        ----------
        e : MPA
            [$E$] Modulus of elasticity [$MPa$].
        i : MM4
            [$I$] Moment of inertia [$mm^4$].
        beta_d : DIMENSIONLESS
            [$\beta_D$] Reduction factor, see 6.4 [$-$].
        l : MM
            [$l$] the buckling length, determined according to Figure 5-2 for a free or partially fixed earth
            support or according to Figure 5-3 for a fixed earth support. [$mm$].
        """
        super().__init__()
        self.e = e
        self.i = i
        self.beta_d = beta_d
        self.l = l

    @staticmethod
    def _evaluate(
        e: MPA,
        i: MM4,
        beta_d: DIMENSIONLESS,
        l: MM,  # noqa: E741
    ) -> N:
        """Evaluates the formula, for more information see the __init__ method."""
        raise_if_negative(
            e=e,
            i=i,
            beta_d=beta_d,
        )
        raise_if_less_or_equal_to_zero(l=l)

        return (e * i * beta_d * (np.pi**2)) / (l**2)

    def latex(self, n: int = 3) -> LatexFormula:
        """Returns LatexFormula object for formula 5.12."""
        return LatexFormula(
            return_symbol=r"N_{cr}",
            result=f"{self:.{n}f}",
            equation=r"\frac{E \cdot I \cdot \beta_D \cdot \pi^2}{l^2}",
            numeric_equation=rf"\frac{{{self.e:.{n}f} \cdot {self.i:.{n}f} \cdot {self.beta_d:.{n}f} \cdot \pi^2}}{{{self.l:.{n}f}^2}}",
            comparison_operator_label="=",
            unit="N",
        )



"""Testing formula 7.11 of EN 1992-1-1:2004."""

import pytest

from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_7_serviceability_limit_state.formula_7_11 import Form7Dot11MaximumCrackSpacing
from blueprints.validations import LessOrEqualToZeroError, NegativeValueError


class TestForm7Dot11MaximumCrackSpacing:
    """Validation for formula 7.11 from EN 1992-1-1:2004."""

    def test_evaluation(self) -> None:
        """Tests the evaluation of the result."""
        # Example values
        k_3 = 0.8
        c = 30.0
        k_1 = 0.5
        k_2 = 1.0
        k_4 = 0.6
        diam = 16.0
        rho_p_eff = 0.02

        # Object to test
        formula = Form7Dot11MaximumCrackSpacing(k_3=k_3, c=c, k_1=k_1, k_2=k_2, k_4=k_4, diam=diam, rho_p_eff=rho_p_eff)

        # Expected result, manually calculated
        manually_calculated_result = 264.000  # mm

        assert formula == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    @pytest.mark.parametrize(
        ("k_3", "c", "k_1", "k_2", "k_4", "diam", "rho_p_eff"),
        [
            (-0.8, 30.0, 0.5, 1.0, 0.6, 16.0, 0.02),  # k_3 is negative
            (0.8, -30.0, 0.5, 1.0, 0.6, 16.0, 0.02),  # c is negative
            (0.8, 30.0, -0.5, 1.0, 0.6, 16.0, 0.02),  # k_1 is negative
            (0.8, 30.0, 0.5, -1.0, 0.6, 16.0, 0.02),  # k_2 is negative
            (0.8, 30.0, 0.5, 1.0, -0.6, 16.0, 0.02),  # k_4 is negative
            (0.8, 30.0, 0.5, 1.0, 0.6, -16.0, 0.02),  # diam is negative
            (0.8, 30.0, 0.5, 1.0, 0.6, 16.0, 0.0),  # rho_p_eff is zero
            (0.8, 30.0, 0.5, 1.0, 0.6, 16.0, -0.02),  # rho_p_eff is negative
        ],
    )
    def test_raise_error_when_invalid_values_are_given(
        self, k_3: float, c: float, k_1: float, k_2: float, k_4: float, diam: float, rho_p_eff: float
    ) -> None:
        """Test invalid values."""
        with pytest.raises((NegativeValueError, LessOrEqualToZeroError)):
            Form7Dot11MaximumCrackSpacing(k_3=k_3, c=c, k_1=k_1, k_2=k_2, k_4=k_4, diam=diam, rho_p_eff=rho_p_eff)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"s_{r,max} = k_3 \cdot c + k_1 \cdot k_2 \cdot k_4 \cdot \frac{⌀}{\rho_{p,eff}} = "
                r"0.800 \cdot 30.000 + 0.500 \cdot 1.000 \cdot 0.600 \cdot \frac{16.000}{0.020} = 264.000 \ mm",
            ),
            ("short", r"s_{r,max} = 264.000 \ mm"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        k_3 = 0.8
        c = 30.0
        k_1 = 0.5
        k_2 = 1.0
        k_4 = 0.6
        diam = 16.0
        rho_p_eff = 0.02

        # Object to test
        latex = Form7Dot11MaximumCrackSpacing(k_3=k_3, c=c, k_1=k_1, k_2=k_2, k_4=k_4, diam=diam, rho_p_eff=rho_p_eff).latex()

        actual = {
            "complete": latex.complete,
            "short": latex.short,
        }

        assert expected == actual[representation], f"{representation} representation failed."



"""Formula 6.29rho from EN 1993-1-1:2005: Chapter 6 - Ultimate Limit State."""

from blueprints.codes.eurocode.en_1993_1_1_2005 import EN_1993_1_1_2005
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula, latex_replace_symbols
from blueprints.type_alias import DIMENSIONLESS, N
from blueprints.validations import raise_if_less_or_equal_to_zero, raise_if_negative


class Form6Dot29Rho(Formula):
    r"""Class representing formula 6.29rho for the calculation of [$\rho$], where no torsion is present."""

    label = "6.29rho"
    source_document = EN_1993_1_1_2005

    def __init__(
        self,
        v_ed: N,
        v_pl_rd: N,
    ) -> None:
        r"""[$\rho$] Calculation of the reduction factor, where no torsion is present [$\text{dimensionless}$].

        EN 1993-1-1:2005 art.6.2.10(3) - Formula (6.29rho)

        Parameters
        ----------
        v_ed : N
            [$V_{Ed}$] Design shear force [$N$].
        v_pl_rd : N
            [$V_{pl,Rd}$] Plastic shear resistance, obtained from 6.2.6(2) [$N$].

        Note, see also 6.2.10(3)
        """
        super().__init__()
        self.v_ed = v_ed
        self.v_pl_rd = v_pl_rd

    @staticmethod
    def _evaluate(
        v_ed: N,
        v_pl_rd: N,
    ) -> DIMENSIONLESS:
        """Evaluates the formula, for more information see the __init__ method."""
        raise_if_less_or_equal_to_zero(v_pl_rd=v_pl_rd)
        raise_if_negative(v_ed=v_ed)

        if v_ed <= 0.5 * v_pl_rd:
            return 0
        return ((2 * v_ed / v_pl_rd) - 1) ** 2

    def latex(self, n: int = 3) -> LatexFormula:
        """Returns LatexFormula object for formula 6.29rho."""
        _equation: str = (
            r"\begin{cases} "
            r"0 & \text{if } V_{Ed} \leq 0.5 \cdot V_{pl,Rd} \\ "
            r"\left( \frac{2 \cdot V_{Ed}}{V_{pl,Rd}} - 1 \right)^2 & \text{if } V_{Ed} > 0.5 \cdot V_{pl,Rd} "
            r"\end{cases}"
        )
        _numeric_equation: str = latex_replace_symbols(
            _equation,
            {
                r"V_{Ed}": f"{self.v_ed:.{n}f}",
                r"V_{pl,Rd}": f"{self.v_pl_rd:.{n}f}",
            },
            False,
        )
        _numeric_equation_with_units: str = latex_replace_symbols(
            _equation,
            {
                r"V_{Ed}": rf"{self.v_ed:.{n}f} \ N",
                r"V_{pl,Rd}": rf"{self.v_pl_rd:.{n}f} \ N",
            },
            False,
        )
        return LatexFormula(
            return_symbol=r"\rho",
            result=f"{self:.{n}f}",
            equation=_equation,
            numeric_equation=_numeric_equation,
            numeric_equation_with_units=_numeric_equation_with_units,
            comparison_operator_label="=",
            unit="-",
        )


class Form6Dot29RhoWithTorsion(Formula):
    r"""Class representing formula 6.29rho with torsion for the calculation of [$\rho$]."""

    label = "6.29rho_with_torsion"
    source_document = EN_1993_1_1_2005

    def __init__(
        self,
        v_ed: N,
        v_pl_t_rd: N,
    ) -> None:
        r"""[$\rho$] Calculation of the reduction factor with torsion [$\text{dimensionless}$].

        EN 1993-1-1:2005 art.6.2.7(4) - Formula (6.29rho with torsion)

        Parameters
        ----------
        v_ed : N
            [$V_{Ed}$] Design shear force [$N$].
        v_pl_t_rd : N
            [$V_{pl,T,Rd}$] Plastic shear resistance with torsion [$N$].

        Note, see also 6.2.7
        """
        super().__init__()
        self.v_ed = v_ed
        self.v_pl_t_rd = v_pl_t_rd

    @staticmethod
    def _evaluate(
        v_ed: N,
        v_pl_t_rd: N,
    ) -> DIMENSIONLESS:
        """Evaluates the formula, for more information see the __init__ method."""
        raise_if_less_or_equal_to_zero(v_pl_t_rd=v_pl_t_rd)
        raise_if_negative(v_ed=v_ed)

        if v_ed <= 0.5 * v_pl_t_rd:
            return 0
        return ((2 * v_ed / v_pl_t_rd) - 1) ** 2

    def latex(self, n: int = 3) -> LatexFormula:
        """Returns LatexFormula object for formula 6.29rho with torsion."""
        _equation: str = (
            r"\begin{cases} "
            r"0 & \text{if } V_{Ed} \leq 0.5 \cdot V_{pl,T,Rd} \\ "
            r"\left( \frac{2 \cdot V_{Ed}}{V_{pl,T,Rd}} - 1 \right)^2 & \text{if } V_{Ed} > 0.5 \cdot V_{pl,T,Rd} "
            r"\end{cases}"
        )
        _numeric_equation: str = latex_replace_symbols(
            _equation,
            {
                r"V_{Ed}": f"{self.v_ed:.{n}f}",
                r"V_{pl,T,Rd}": f"{self.v_pl_t_rd:.{n}f}",
            },
            False,
        )
        _numeric_equation_with_units: str = latex_replace_symbols(
            _equation,
            {
                r"V_{Ed}": rf"{self.v_ed:.{n}f} \ N",
                r"V_{pl,T,Rd}": rf"{self.v_pl_t_rd:.{n}f} \ N",
            },
            False,
        )
        return LatexFormula(
            return_symbol=r"\rho",
            result=f"{self:.{n}f}",
            equation=_equation,
            numeric_equation=_numeric_equation,
            numeric_equation_with_units=_numeric_equation_with_units,
            comparison_operator_label="=",
            unit="-",
        )



"""Testing formula 6.58/6.59 of EN 1992-1-1:2004."""

import pytest

from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_6_ultimate_limit_state.formula_6_58_59 import Form6Dot58And59TensileForce
from blueprints.validations import LessOrEqualToZeroError, NegativeValueError


class TestForm6Dot58And59TensileForce:
    """Validation for formula 6.58/6.59 from EN 1992-1-1:2004."""

    def test_evaluation58(self) -> None:
        """Tests the evaluation of the result."""
        # Example values
        f = 500.0
        a = 50.0
        b = 100.0
        capital_h = 400.0

        # Object to test
        formula = Form6Dot58And59TensileForce(f=f, a=a, b=b, capital_h=capital_h)

        # Expected result, manually calculated
        manually_calculated_result = 62.5

        assert formula == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    def test_evaluation59(self) -> None:
        """Tests the evaluation of the result."""
        # Example values
        f = 500.0
        a = 200.0
        b = 300.0
        capital_h = 400.0

        # Object to test
        formula = Form6Dot58And59TensileForce(f=f, a=a, b=b, capital_h=capital_h)

        # Expected result, manually calculated
        manually_calculated_result = 37.500

        assert formula == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    @pytest.mark.parametrize(
        ("f", "a", "b", "capital_h"),
        [
            (-500.0, 200.0, 300.0, 400.0),  # f is negative
            (500.0, -200.0, 300.0, 400.0),  # a is negative
            (500.0, 200.0, -300.0, 400.0),  # b is negative
            (500.0, 200.0, 300.0, -400.0),  # capital_h is negative
            (500.0, 200.0, 0.0, 400.0),  # b is zero
            (500.0, 200.0, 300.0, 0.0),  # capital_h is zero
        ],
    )
    def test_raise_error_when_invalid_values_are_given(self, f: float, a: float, b: float, capital_h: float) -> None:
        """Test invalid values."""
        with pytest.raises((NegativeValueError, LessOrEqualToZeroError)):
            Form6Dot58And59TensileForce(f=f, a=a, b=b, capital_h=capital_h)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"T = \begin{cases} \frac{1}{4} \cdot \frac{ b - a}{ b} \cdot F & \text{if } b \leq \frac{H}{2} \\ "
                r"\frac{1}{4} \cdot \left(1 - 0.7 \cdot \frac{ a}{\frac{H}{2}}\right) \cdot F & \text{if } b > \frac{H}{2} \end{cases}"
                r" = \begin{cases} \frac{1}{4} \cdot \frac{ 300.000 - 200.000}{ 300.000} \cdot 500.000 & \text{if } 300.000 "
                r"\leq \frac{400.000}{2} \\ \frac{1}{4} \cdot \left(1 - 0.7 \cdot \frac{ 200.000}{\frac{400.000}{2}}\right) \cdot "
                r"500.000 & \text{if } 300.000 > \frac{400.000}{2} \end{cases} = 37.500 \ N",
            ),
            ("short", r"T = 37.500 \ N"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        f = 500.0
        a = 200.0
        b = 300.0
        capital_h = 400.0

        # Object to test
        latex = Form6Dot58And59TensileForce(f=f, a=a, b=b, capital_h=capital_h).latex()

        actual = {
            "complete": latex.complete,
            "short": latex.short,
        }

        assert expected == actual[representation], f"{representation} representation failed."



"""Testing formula 5.15 of EN 1992-1-1:2004."""

import pytest

from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_5_structural_analysis.formula_5_15 import Form5Dot15EffectiveLengthBraced
from blueprints.validations import NegativeValueError


class TestForm5Dot15EffectiveLengthBraced:
    """Validation for formula 5.15 from EN 1992-1-1:2004."""

    def test_evaluation(self) -> None:
        """Test the evaluation of the result."""
        # Example values
        k_1 = 2.0  # -
        k_2 = 3.0  # -
        height = 4.0  # M

        # Object to test
        form_5_15 = Form5Dot15EffectiveLengthBraced(k_1=k_1, k_2=k_2, height=height)

        # Expected result, manually calculated
        manually_calculated_result = 3.6855  # M

        assert form_5_15 == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    @pytest.mark.parametrize(
        ("k_1", "k_2", "height"),
        [
            (-2, 3, 4),
            (2, -3, 4),
            (2, 3, -4),
        ],
    )
    def test_raise_error_when_negative_theta_i_is_given(self, k_1: float, k_2: float, height: float) -> None:
        """Test negative values for theta_i, n_a and n_b."""
        with pytest.raises(NegativeValueError):
            Form5Dot15EffectiveLengthBraced(k_1=k_1, k_2=k_2, height=height)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"l_0 = 0.5 \cdot l \cdot \sqrt{"
                r"\left(1+\frac{k_1}{0.45 + k_1}\right) \cdot "
                r"\left(1 + \frac{k_2}{0.45 + k_2}\right)} = "
                r"0.5 \cdot 4.000 \cdot \sqrt{"
                r"\left(1+\frac{2.000}{0.45 + 2.000}\right) \cdot "
                r"\left(1 + \frac{3.000}{0.45 + 3.000}\right)} = "
                r"3.686",
            ),
            ("short", r"l_0 = 3.686"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        k_1 = 2.0  # -
        k_2 = 3.0  # -
        height = 4.0  # M

        # Object to test
        form_5_15_latex = Form5Dot15EffectiveLengthBraced(k_1=k_1, k_2=k_2, height=height).latex()

        actual = {
            "complete": form_5_15_latex.complete,
            "short": form_5_15_latex.short,
        }

        assert actual[representation] == expected, f"{representation} representation failed."



"""Formula 6.19 from EN 1992-1-1:2004: Chapter 6 - Ultimate Limit State."""

import numpy as np

from blueprints.codes.eurocode.en_1992_1_1_2004 import EN_1992_1_1_2004
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula
from blueprints.type_alias import DEG, MM2, MPA, N
from blueprints.validations import raise_if_negative


class Form6Dot19CheckShearForce(Formula):
    r"""Class representing formula 6.19 for checking the shear force."""

    label = "6.19"
    source_document = EN_1992_1_1_2004

    def __init__(
        self,
        v_ed: N,
        a_sw: MM2,
        f_ywd: MPA,
        alpha: DEG,
    ) -> None:
        r"""Check the shear force with :math:`V_{Ed} \leq A_{sw} \cdot f_{ywd} \cdot \sin(\alpha)`.

        EN 1992-1-1:2004 art.6.2.3(8) - Formula (6.19)

        Parameters
        ----------
        v_ed : N
            [:math:`V_{Ed}`] Design value of the shear force [:math:`N`].
        a_sw : MM2
            [:math:`A_{sw}`] Cross-sectional area of the shear reinforcement [:math:`mm^2`].
        f_ywd : MPA
            [:math:`f_{ywd}`] Design yield strength of the shear reinforcement [:math:`MPa`].
        alpha : DEG
            [:math:`\alpha`] Angle between shear reinforcement and the beam axis perpendicular to the shear force [:math:`degrees`].
        """
        super().__init__()
        self.v_ed = v_ed
        self.a_sw = a_sw
        self.f_ywd = f_ywd
        self.alpha = alpha

    @staticmethod
    def _evaluate(
        v_ed: N,
        a_sw: MM2,
        f_ywd: MPA,
        alpha: DEG,
    ) -> bool:
        """Evaluates the formula, for more information see the __init__ method."""
        raise_if_negative(alpha=alpha, f_ywd=f_ywd, a_sw=a_sw, v_ed=v_ed)

        return v_ed <= a_sw * f_ywd * np.sin(np.deg2rad(alpha))

    def latex(self, n: int = 3) -> LatexFormula:
        """Returns LatexFormula object for formula 6.19."""
        return LatexFormula(
            return_symbol=r"CHECK",
            result="OK" if self.__bool__() else "\\text{Not OK}",
            equation=r"V_{Ed} \leq A_{sw} \cdot f_{ywd} \cdot \sin(\alpha)",
            numeric_equation=rf"{self.v_ed:.{n}f} \leq {self.a_sw:.{n}f} \cdot {self.f_ywd:.{n}f} \cdot \sin({self.alpha:.{n}f})",
            comparison_operator_label="\\to",
            unit="",
        )



"""Rebar class for the representation of reinforcement bars in a cross-section."""

from dataclasses import dataclass

from blueprints.materials.reinforcement_steel import ReinforcementSteelMaterial
from blueprints.structural_sections.cross_section_circle import CircularCrossSection
from blueprints.type_alias import KG_M, RATIO
from blueprints.unit_conversion import MM2_TO_M2


@dataclass(frozen=True, kw_only=True)
class Rebar(CircularCrossSection):
    """
    Representation of a reinforcement bar from a cross-section perspective. For example ⌀16, ⌀20, ⌀25, ⌀32,etc.

    Parameters
    ----------
    diameter : MM
        Diameter of the bar (for example: ⌀12, ⌀16, ⌀20, etc.) [mm]
    x : MM
        x-coordinate in the cross-section [mm]
    y : MM
        y-coordinate in the cross-section [mm]
    material : ReinforcementSteelMaterial
        Representation of the properties of reinforcement steel suitable for use with NEN-EN 1992-1-1.
    relative_start_position: RATIO
        Relative position of the start of the rebar in the longitudinal direction of the host element [-]
    relative_end_position: RATIO
        Relative position of the end of the rebar in the longitudinal direction of the host element [-]
    name: str
        Name of the rebar element, default is "Rebar"
    """

    material: ReinforcementSteelMaterial
    relative_start_position: RATIO = 0.0
    relative_end_position: RATIO = 1.0
    name: str = "Rebar"

    def __post_init__(self) -> None:
        """Post-initialization to validate the diameter."""
        super().__post_init__()
        if not 0.0 <= self.relative_end_position <= 1.0:
            msg = f"Relative end position of the rebar must be between 0.0 and 1.0, but got {self.relative_end_position}"
            raise ValueError(msg)

        if not 0.0 <= self.relative_start_position <= 1.0:
            msg = f"Relative start position of the rebar must be between 0.0 and 1.0, but got {self.relative_start_position}"
            raise ValueError(msg)

    @property
    def weight_per_meter(self) -> KG_M:
        """Unit weight of rebar per meter [kg/m].

        Returns
        -------
        float
            Example: 1.578336149163512 for ⌀16 and normal density of 7850 kg/m3
        """
        return self.material.density * (self.area * MM2_TO_M2)



"""Formula 6.3N from EN 1992-1-1:2004: Chapter 6 - Ultimate Limit State."""

from blueprints.codes.eurocode.en_1992_1_1_2004 import EN_1992_1_1_2004
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula
from blueprints.type_alias import DIMENSIONLESS, MPA
from blueprints.validations import raise_if_negative


class Form6Dot3NShearCapacityWithoutRebar(Formula):
    r"""Class representing formula 6.3N for the calculation of the shear capacity without rebar, [$v_{min}$]."""

    label = "6.3N"
    source_document = EN_1992_1_1_2004

    def __init__(
        self,
        k: DIMENSIONLESS,
        f_ck: MPA,
    ) -> None:
        r"""[$v_{min}$] Shear capacity without rebar [$MPa$].

        EN 1992-1-1:2004 art.6.2.2(1) - Formula (6.3N)

        Parameters
        ----------
        k : DIMENSIONLESS
            [$k$] Factor which depends on the thickness concrete, see formula 6.2 [$-$].
        f_ck : MPA
            [$f_{ck}$] Characteristic compressive strength of concrete [$MPa$].
        """
        super().__init__()
        self.k = k
        self.f_ck = f_ck

    @staticmethod
    def _evaluate(
        k: DIMENSIONLESS,
        f_ck: MPA,
    ) -> MPA:
        """Evaluates the formula, for more information see the __init__ method."""
        raise_if_negative(k=k, f_ck=f_ck)

        return 0.035 * k**1.5 * f_ck**0.5

    def latex(self, n: int = 3) -> LatexFormula:
        """Returns LatexFormula object for formula 6.3N."""
        return LatexFormula(
            return_symbol=r"v_{min}",
            result=f"{self:.{n}f}",
            equation=r"0.035 \cdot k^{3/2} \cdot f_{ck}^{1/2}",
            numeric_equation=rf"0.035 \cdot {self.k:.{n}f}^{{3/2}} \cdot {self.f_ck:.{n}f}^{{1/2}}",
            comparison_operator_label="=",
            unit="MPa",
        )



"""Formula 7.1 from EN 1992-1-1:2004: Chapter 7 - Serviceability Limit State."""

from blueprints.codes.eurocode.en_1992_1_1_2004 import EN_1992_1_1_2004
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula, latex_replace_symbols
from blueprints.type_alias import DIMENSIONLESS, MM2, MPA
from blueprints.validations import raise_if_less_or_equal_to_zero, raise_if_negative


class Form7Dot1MinReinforcingSteel(Formula):
    r"""Class representing formula 7.1 for the calculation of [$A_{s,min}$]."""

    label = "7.1"
    source_document = EN_1992_1_1_2004

    def __init__(
        self,
        k_c: DIMENSIONLESS,
        k: DIMENSIONLESS,
        f_ct_eff: MPA,
        a_ct: MM2,
        sigma_s: MPA,
    ) -> None:
        r"""[$A_{s,min}$] Calculation of minimum area of reinforcing steel within the tensile zone.

        EN 1992-1-1:2004 art.7.3.2(2) - Formula (7.1)

        Parameters
        ----------
        k_c : DIMENSIONLESS
            [$k_c$] Coefficient which takes account of the stress distribution within the section immediately
            prior to cracking and of the change of the lever arm. For pure tension use 1.0, else use equation 7.2 or 7.3 [$-$].
        k : DIMENSIONLESS
            [$k$] Coefficient which allows for the effect of non-uniform self-equilibrating stresses, which lead to a
            reduction of restraint forces [$-$].
        f_ct_eff : MPA
            [$f_{ct,eff}$] Mean value of the tensile strength of the concrete effective at the time when the cracks may
            first be expected to occur [$MPa$].
        a_ct : MM2
            [$A_{ct}$] Area of concrete within the tensile zone [$mm^2$].
        sigma_s : MPA
            [$\sigma_s$] Absolute value of the maximum stress permitted in the reinforcement immediately after formation
            of the crack [$MPa$].
        """
        super().__init__()
        self.k_c = k_c
        self.k = k
        self.f_ct_eff = f_ct_eff
        self.a_ct = a_ct
        self.sigma_s = sigma_s

    @staticmethod
    def _evaluate(
        k_c: DIMENSIONLESS,
        k: DIMENSIONLESS,
        f_ct_eff: MPA,
        a_ct: MM2,
        sigma_s: MPA,
    ) -> MM2:
        """Evaluates the formula, for more information see the __init__ method."""
        raise_if_negative(k_c=k_c, k=k, f_ct_eff=f_ct_eff, a_ct=a_ct)
        raise_if_less_or_equal_to_zero(sigma_s=sigma_s)

        return (k_c * k * f_ct_eff * a_ct) / sigma_s

    def latex(self, n: int = 3) -> LatexFormula:
        """Returns LatexFormula object for formula 7.1."""
        _equation: str = r"\frac{k_c \cdot k \cdot f_{ct,eff} \cdot A_{ct}}{\sigma_s}"
        _numeric_equation: str = latex_replace_symbols(
            _equation,
            {
                r"k_c": f"{self.k_c:.{n}f}",
                r"k": f"{self.k:.{n}f}",
                r"f_{ct,eff}": f"{self.f_ct_eff:.{n}f}",
                r"A_{ct}": f"{self.a_ct:.{n}f}",
                r"\sigma_s": f"{self.sigma_s:.{n}f}",
            },
            False,
        )
        return LatexFormula(
            return_symbol=r"A_{s,min}",
            result=f"{self:.{n}f}",
            equation=_equation,
            numeric_equation=_numeric_equation,
            comparison_operator_label="=",
            unit="mm^2",
        )



"""Formula 5.13N from EN 1992-1-1:2004: Chapter 5 Structural Analysis."""

import numpy as np

from blueprints.codes.eurocode.en_1992_1_1_2004 import EN_1992_1_1_2004
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula, latex_replace_symbols
from blueprints.type_alias import DIMENSIONLESS, KNM, MM2, MPA, N
from blueprints.validations import EqualToZeroError, raise_if_less_or_equal_to_zero, raise_if_negative


class Form5Dot13nSlendernessCriterionIsolatedMembers(Formula):
    """Class representing formula 5.13N for the calculation of the slenderness limit where second order effects may be ignored."""

    label = "5.13N"
    source_document = EN_1992_1_1_2004

    def __init__(self, a: DIMENSIONLESS, b: DIMENSIONLESS, c: DIMENSIONLESS, n_ed: N, a_c: MM2, f_cd: MPA) -> None:
        """[$λ_{lim}$] Calculation of the slenderness limit, where second order effects may be ignored (dimensionless).

        Note:
        The value of [$λ_{lim}$] for use in a Country may be found in its National Annex. The recommended value
        follows from this equation.

        EN 1992-1-1:2004 art.5.8.3.1 (1) - formula (5.13N)

        Parameters
        ----------
        a : DIMENSIONLESS
            [$A$] calculation value, based on the effective creep ratio [$-$].
            Follows from equation 5.13A. If unknown, A = 0,7.
            Use your own implementation of this value or use the `SubForm5Dot13aCreepRatio` class.
        b : DIMENSIONLESS
            [$B$] calculation value, based on the mechanical reinforcement ratio [$-$].
            Follows from equation 5.13B. If unknown, B = 1,1.
            Use your own implementation of this value or use the `SubForm5Dot13bMechanicalReinforcementFactor` class.
        c : DIMENSIONLESS
            [$C$] calculation value, based on the moment ratio [$-$].
            Follows from equation 5.13C. If unknown, C = 0,7.
            Use your own implementation of this value or use the `SubForm5Dot13cMomentRatio` class.
        n_ed : N
            [$N_{Ed}$] is the design value of the axial force [$N$].
        a_c : MM2
            [$A_c$] is the area of the concrete section [$mm^2$].
        f_cd :
            [$f_{cd}$] is the design value of concrete compressive strength [$MPa$].
        """
        super().__init__()
        self.a = a
        self.b = b
        self.c = c
        self.n_ed = n_ed
        self.a_c = a_c
        self.f_cd = f_cd

    @staticmethod
    def _evaluate(a: DIMENSIONLESS, b: DIMENSIONLESS, c: DIMENSIONLESS, n_ed: N, a_c: MM2, f_cd: MPA) -> DIMENSIONLESS:
        """Evaluates the formula, for more information see the __init__ method."""
        raise_if_negative(a=a, b=b, c=c)
        raise_if_less_or_equal_to_zero(a_c=a_c, f_cd=f_cd, n_ed=n_ed)

        n = n_ed / (a_c * f_cd)
        return (20 * a * b * c) / np.sqrt(n)

    def latex(self, n: int = 3) -> LatexFormula:
        """Returns LatexFormula object for formula 5.13N."""
        _equation: str = r"\frac{20 \cdot A \cdot B \cdot C}{\sqrt{N_{Ed} \cdot A_c \cdot f_{cd}}}"
        _numeric_equation: str = latex_replace_symbols(
            _equation,
            {
                r"A_c": f"{self.a_c:.{n}f}",
                r"A": f"{self.a:.{n}f}",
                r"B": f"{self.b:.{n}f}",
                r"C": f"{self.c:.{n}f}",
                r"N_{Ed}": f"{self.n_ed:.{n}f}",
                r"f_{cd}": f"{self.f_cd:.{n}f}",
            },
            True,
        )

        _numeric_equation_with_units: str = latex_replace_symbols(
            _equation,
            {
                r"A_c": rf"{self.a_c:.{n}f} \ mm^2",
                r"A": f"{self.a:.{n}f}",
                r"B": f"{self.b:.{n}f}",
                r"C": f"{self.c:.{n}f}",
                r"N_{Ed}": rf"{self.n_ed:.{n}f} \ N",
                r"f_{cd}": rf"{self.f_cd:.{n}f} \ MPa",
            },
        )
        return LatexFormula(
            return_symbol=r"\lambda_{lim}",
            result=f"{self:.{n}f}",
            equation=_equation,
            numeric_equation=_numeric_equation,
            numeric_equation_with_units=_numeric_equation_with_units,
            comparison_operator_label="=",
            unit="-",
        )


class SubForm5Dot13aCreepRatio(Formula):
    r"""Class representing sub-formula for [$A$] in formula (5.13N)."""

    label = "5.13"
    source_document = EN_1992_1_1_2004

    def __init__(
        self,
        phi_ef: DIMENSIONLESS,
    ) -> None:
        r"""[$A$] Calculation of the factor [$A$] in formula (5.13N).

        EN 1992-1-1:2004 art.5.8.3.1 (1) - formula (5.13N)

        Parameters
        ----------
        phi_ef : DIMENSIONLESS
            [$\phi_{ef}$] Effective creep ratio; see 5.8.4 (If [$\phi_{ef}$] is not known, A = 0.7)
        """
        super().__init__()
        self.phi_ef = phi_ef

    @staticmethod
    def _evaluate(
        phi_ef: DIMENSIONLESS,
    ) -> DIMENSIONLESS:
        """Evaluates the formula, for more information see the __init__ method."""
        raise_if_negative(phi_ef=phi_ef)

        return 1 / (1 + 0.2 * phi_ef)

    def latex(self, n: int = 3) -> LatexFormula:
        """Returns LatexFormula object for formula 5.13a."""
        _equation: str = r"\frac{1}{(1 + 0.2 \cdot \phi_{ef})}"
        _numeric_equation: str = latex_replace_symbols(
            _equation,
            {
                r"\phi_{ef}": f"{self.phi_ef:.{n}f}",
            },
            False,
        )
        return LatexFormula(
            return_symbol=r"A",
            result=f"{self:.{n}f}",
            equation=_equation,
            numeric_equation=_numeric_equation,
            comparison_operator_label="=",
            unit="-",
        )


class SubForm5Dot13bMechanicalReinforcementFactor(Formula):
    r"""Class representing sub-formula for [$B$] in formula (5.13N)."""

    label = "5.13"
    source_document = EN_1992_1_1_2004

    def __init__(
        self,
        a_s: MM2,
        f_yd: MPA,
        a_c: MM2,
        f_cd: MPA,
    ) -> None:
        r"""[$B$] Calculation of the factor [$B$] in formula (5.13N).

        EN 1992-1-1:2004 art.5.8.3.1 (1) - formula (5.13N)

        Parameters
        ----------
        a_s : MM2
            [$A_s$] is the total area of longitudinal reinforcement [$mm^2$].
        f_yd : MPA
            [$f_{yd}$] is the design yield stress of the reinforcement [$MPa$].
        a_c : MM2
            [$A_c$] is the area of concrete section [$mm^2$].
        f_cd : MPA
            [$f_{cd}$] is the design value of concrete compressive strength [$MPa$].
        """
        super().__init__()
        self.a_s = a_s
        self.f_yd = f_yd
        self.a_c = a_c
        self.f_cd = f_cd

    @staticmethod
    def _evaluate(
        a_s: MM2,
        f_yd: MPA,
        a_c: MM2,
        f_cd: MPA,
    ) -> DIMENSIONLESS:
        """Evaluates the formula, for more information see the __init__ method."""
        raise_if_negative(a_s=a_s, f_yd=f_yd)
        raise_if_less_or_equal_to_zero(a_c=a_c, f_cd=f_cd)

        mechanical_reinforcement_ratio = (a_s * f_yd) / (a_c * f_cd)
        return np.sqrt(1 + 2 * mechanical_reinforcement_ratio)

    def latex(self, n: int = 3) -> LatexFormula:
        """Returns LatexFormula object for formula 5.13b."""
        _equation: str = r"\sqrt{1 + 2 \cdot \frac{A_s \cdot f_{yd}}{A_c \cdot f_{cd}}}"
        _numeric_equation: str = latex_replace_symbols(
            _equation,
            {
                r"A_s": f"{self.a_s:.{n}f}",
                r"f_{yd}": f"{self.f_yd:.{n}f}",
                r"A_c": f"{self.a_c:.{n}f}",
                r"f_{cd}": f"{self.f_cd:.{n}f}",
            },
            False,
        )
        _numeric_equation_with_units: str = latex_replace_symbols(
            _equation,
            {
                r"A_s": rf"{self.a_s:.{n}f} \ mm^2",
                r"f_{yd}": rf"{self.f_yd:.{n}f} \ MPa",
                r"A_c": rf"{self.a_c:.{n}f} \ mm^2",
                r"f_{cd}": rf"{self.f_cd:.{n}f} \ MPa",
            },
            False,
        )
        return LatexFormula(
            return_symbol=r"B",
            result=f"{self:.{n}f}",
            equation=_equation,
            numeric_equation=_numeric_equation,
            numeric_equation_with_units=_numeric_equation_with_units,
            comparison_operator_label="=",
            unit="-",
        )


class SubForm5Dot13cMomentRatio(Formula):
    r"""Class representing sub-formula for [$C$] in formula (5.13N)."""

    label = "5.13c"
    source_document = EN_1992_1_1_2004

    def __init__(
        self,
        m_01: KNM,
        m_02: KNM,
    ) -> None:
        r"""[$C$] Calculation of the factor [$C$] in formula (5.13N).

        EN 1992-1-1:2004 art.5.8.3.1 (1) - formula (5.13)

        Parameters
        ----------
        m_01 : KNM
            [$M_{01}$] is one of the first order end moments [$kNm$].
        m_02 : KNM
            [$M_{02}$] is one of the first order end moments [$kNm$].
        """
        super().__init__()
        self.m_01 = m_01
        self.m_02 = m_02

    @staticmethod
    def _evaluate(
        m_01: KNM,
        m_02: KNM,
    ) -> DIMENSIONLESS:
        """Evaluates the formula, for more information see the __init__ method."""
        if m_02 == 0.0:
            raise EqualToZeroError(value_name="m_02", value=m_02)

        if not abs(m_02) >= abs(m_01):
            raise ValueError("The absolute value of M_02 must be equal to or larger than the absolute value of M_01.")

        return 1.7 - (m_01 / m_02)

    def latex(self, n: int = 3) -> LatexFormula:
        """Returns LatexFormula object for formula 5.13b."""
        _equation: str = r"1.7 - \frac{M_{01}}{M_{02}}"
        _numeric_equation: str = latex_replace_symbols(
            _equation,
            {
                r"M_{01}": f"{self.m_01:.{n}f}",
                r"M_{02}": f"{self.m_02:.{n}f}",
            },
            False,
        )
        _numeric_equation_with_units: str = latex_replace_symbols(
            _equation,
            {
                r"M_{01}": rf"{self.m_01:.{n}f} \ kNm",
                r"M_{02}": rf"{self.m_02:.{n}f} \ kNm",
            },
        )
        return LatexFormula(
            return_symbol=r"C",
            result=f"{self:.{n}f}",
            equation=_equation,
            numeric_equation=_numeric_equation,
            numeric_equation_with_units=_numeric_equation_with_units,
            comparison_operator_label="=",
            unit="-",
        )



"""Testing formula 5.9 of EN 1992-1-1:2004."""

import pytest

from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_5_structural_analysis.formula_5_9 import Form5Dot9DesignSupportMomentReduction
from blueprints.validations import NegativeValueError


class TestForm5Dot9DesignSupportMomentReduction:
    """Validation for formula 5.9 from EN 1992-1-1:2004."""

    def test_evaluation(self) -> None:
        """Tests the evaluation of the result."""
        # Example values
        f_ed_sup = 100
        t = 0.3

        # Object to test
        formula = Form5Dot9DesignSupportMomentReduction(f_ed_sup=f_ed_sup, t=t)

        # Expected result, manually calculated
        manually_calculated_result = 3.75  # kNm

        assert formula == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    @pytest.mark.parametrize(
        ("f_ed_sup", "t"),
        [
            (-100, 0.3),
            (100, -0.3),
        ],
    )
    def test_raise_error_when_negative_values_are_given(self, f_ed_sup: float, t: float) -> None:
        """Test negative values."""
        with pytest.raises(NegativeValueError):
            Form5Dot9DesignSupportMomentReduction(f_ed_sup=f_ed_sup, t=t)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"ΔM_{Ed} = \frac{F_{Ed,sup} \cdot t}{8} = \frac{100 \cdot 0.3}{8} = 3.750",
            ),
            ("short", r"ΔM_{Ed} = 3.750"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        f_ed_sup = 100
        t = 0.3

        # Object to test
        latex = Form5Dot9DesignSupportMomentReduction(f_ed_sup=f_ed_sup, t=t).latex()

        actual = {
            "complete": latex.complete,
            "short": latex.short,
        }

        assert expected == actual[representation], f"{representation} representation failed."



"""Formula 5.31 from EN 1992-1-1:2004: Chapter 5 - Structural Analysis."""

from blueprints.codes.eurocode.en_1992_1_1_2004 import EN_1992_1_1_2004
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula
from blueprints.type_alias import KNM


class Form5Dot31DesignMoment(Formula):
    """Class representing formula 5.31 for the calculation of the design moment, [$M_{Ed}$]."""

    label = "5.31"
    source_document = EN_1992_1_1_2004

    def __init__(
        self,
        m_0ed: KNM,
        m_2: KNM,
    ) -> None:
        r"""[$M_{Ed}$] Design moment [$kNm$].

        EN 1992-1-1:2004 art.5.8.8.2(2) - Formula (5.31)

        Parameters
        ----------
        m_0ed : KNM
            [$M_{0Ed}$] First order moment, including the effect of imperfections; see also 5.8.8.2 (2) [$kNm$].
        m_2 : KNM
            [$M_{2}$] Nominal 2nd order moment; see 5.8.8.2 (3) [$kNm$].
        """
        super().__init__()
        self.m_0ed = m_0ed
        self.m_2 = m_2

    @staticmethod
    def _evaluate(
        m_0ed: KNM,
        m_2: KNM,
    ) -> KNM:
        """Evaluates the formula, for more information see the __init__ method."""
        return m_0ed + m_2

    def latex(self, n: int = 3) -> LatexFormula:
        """Returns LatexFormula object for formula 5.31."""
        return LatexFormula(
            return_symbol=r"M_{Ed}",
            result=f"{self:.{n}f}",
            equation=r"M_{0Ed} + M_{2}",
            numeric_equation=rf"{self.m_0ed:.{n}f} + {self.m_2:.{n}f}",
            comparison_operator_label="=",
            unit="kNm",
        )



"""Testing formula 9.2 of EN 1992-1-1:2004."""

import pytest

from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_9_detailling_and_specific_rules.formula_9_2 import Form9Dot2ShiftInMomentDiagram
from blueprints.validations import GreaterThan90Error, NegativeValueError


class TestForm9Dot2ShiftInMomentDiagram:
    """Validation for formula 9.2 from EN 1992-1-1:2004."""

    def test_evaluation(self) -> None:
        """Test the evaluation of the result."""
        # Example values
        z = 250  # mm
        theta = 30  # deg
        alpha = 85  # deg
        form_9_2 = Form9Dot2ShiftInMomentDiagram(z=z, theta=theta, alpha=alpha)

        # Expected result, manually calculated
        manually_calculated_result = 205.570268

        assert form_9_2 == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    def test_raise_error_when_negative_z_is_given(self) -> None:
        """Test if error is raised when z is negative."""
        # Example values
        z = -250  # mm
        theta = 30  # deg
        alpha = 85  # deg

        with pytest.raises(NegativeValueError):
            Form9Dot2ShiftInMomentDiagram(z=z, theta=theta, alpha=alpha)

    def test_raise_error_when_alpha_is_negative(self) -> None:
        """Test if error is raised when alpha is negative."""
        # Example values
        z = 250  # mm
        theta = 30  # deg
        alpha = -50  # deg

        with pytest.raises(NegativeValueError):
            Form9Dot2ShiftInMomentDiagram(z=z, theta=theta, alpha=alpha)

    def test_raise_error_when_alpha_is_greater_90(self) -> None:
        """Test if error is raised when alpha is greater than 90."""
        # Example values
        z = 250  # mm
        theta = 30  # deg
        alpha = 95  # deg

        with pytest.raises(GreaterThan90Error):
            Form9Dot2ShiftInMomentDiagram(z=z, theta=theta, alpha=alpha)

    def test_raise_error_when_theta_is_negative(self) -> None:
        """Test if error is raised when theta is negative."""
        # Example values
        z = 250  # mm
        theta = -30  # deg
        alpha = 50  # deg

        with pytest.raises(NegativeValueError):
            Form9Dot2ShiftInMomentDiagram(z=z, theta=theta, alpha=alpha)

    def test_raise_error_when_theta_is_greater_90(self) -> None:
        """Test if error is raised when theta is greater than 90."""
        # Example values
        z = 250  # mm
        theta = 95  # deg
        alpha = 85  # deg

        with pytest.raises(GreaterThan90Error):
            Form9Dot2ShiftInMomentDiagram(z=z, theta=theta, alpha=alpha)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                (
                    r"a_l = z \cdot \left( \cot(\theta) - \cot(\alpha) \right) / 2 = "
                    r"250.00 \cdot \left( \cot(30.00) - \cot(85.00) \right) / 2 = 205.57"
                ),
            ),
            ("short", r"a_l = 205.57"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        z = 250  # mm
        theta = 30  # deg
        alpha = 85  # deg

        # Object to test
        form_9_2_latex = Form9Dot2ShiftInMomentDiagram(z=z, theta=theta, alpha=alpha).latex()

        actual = {"complete": form_9_2_latex.complete, "short": form_9_2_latex.short}

        assert actual[representation] == expected, f"{representation} representation failed."



"""Formula 7.6 from EN 1995-1-1:2004."""

from blueprints.codes.eurocode.en_1995_1_1_2004 import EN_1995_1_1_2004
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula, latex_replace_symbols
from blueprints.type_alias import HZ, KG_M2, M_NS2, M
from blueprints.validations import raise_if_less_or_equal_to_zero


class Form7Dot6VelocityResponse(Formula):
    r"""Class representing formula 7.6 for the calculation of velocity response [$v$] of a unit impulse load."""

    label = "7.6"
    source_document = EN_1995_1_1_2004

    def __init__(self, n_40: HZ, m: KG_M2, length: M, b: M) -> None:
        r"""[$v$] The velocity response of a unit impulse load, in [$m/(Ns^2)$].

        EN 1995-1-1:2004 art 7.3.3(5) - Formula (7.6)

        Parameters
        ----------
        n_40 : HZ
            [$n_{40}$] Number of first-order vibrations with a natural frequency less than 40 Hz [$Hz$].
        m : KG_M2
            [$m$] Mass per unit area [$kg/m^{2}$].
        length : M
            [$l$] Span of the floor [$m$].
        b : M
            [$b$] Width of the floor [$m$].

        Returns
        -------
        None
        """
        super().__init__()
        self.n_40 = n_40
        self.m = m
        self.length = length
        self.b = b

    @staticmethod
    def _evaluate(n_40: HZ, m: KG_M2, length: M, b: M) -> M_NS2:
        """Evaluate the formula, for more information see the __init__ method."""
        raise_if_less_or_equal_to_zero(n_40=n_40, m=m, length=length, b=b)
        return 4 * (0.4 + 0.6 * n_40) / (m * b * length + 200)

    def latex(self, n: int = 2) -> LatexFormula:
        """Returns LatexFormula object for formula 7.6."""
        eq_form: str = r"\frac{4 \cdot (0.4 + 0.6 \cdot n_{40})}{m \cdot b \cdot l + 200}"
        repl_symb = {"n_{40}": f"{self.n_40:.{n}f}", "m": f"{self.m:.{n}f}", "l": f"{self.length:.{n}f}", "b": f"{self.b:.{n}f}"}
        return LatexFormula(
            return_symbol=r"v",
            result=f"{self:.{n + 1}f}",
            equation=eq_form,
            numeric_equation=latex_replace_symbols(eq_form, repl_symb),
            comparison_operator_label="=",
        )



"""Testing formula 6.20 of EN 1993-1-1:2005."""

import pytest

from blueprints.codes.eurocode.en_1993_1_1_2005.chapter_6_ultimate_limit_state.formula_6_20 import Form6Dot20ShearStress
from blueprints.validations import LessOrEqualToZeroError, NegativeValueError


class TestForm6Dot20ShearStress:
    """Validation for formula 6.20 from EN 1993-1-1:2005."""

    def test_evaluation(self) -> None:
        """Tests the evaluation of the result."""
        # Example values
        v_ed = 1000.0
        s = 2000.0
        i = 3000.0
        t = 4.0

        # Object to test
        formula = Form6Dot20ShearStress(v_ed=v_ed, s=s, i=i, t=t)

        # Expected result, manually calculated
        manually_calculated_result = 166.66666666666666  # MPa

        assert formula == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    @pytest.mark.parametrize(
        ("v_ed", "s", "i", "t"),
        [
            (-1000.0, 2000.0, 3000.0, 4.0),  # v_ed is negative
            (1000.0, -2000.0, 3000.0, 4.0),  # s is negative
            (1000.0, 2000.0, 0.0, 4.0),  # i is zero
            (1000.0, 2000.0, 3000.0, 0.0),  # t is zero
            (1000.0, 2000.0, -3000.0, 4.0),  # i is negative
            (1000.0, 2000.0, 3000.0, -4.0),  # t is negative
        ],
    )
    def test_raise_error_when_invalid_values_are_given(self, v_ed: float, s: float, i: float, t: float) -> None:
        """Test invalid values."""
        with pytest.raises((NegativeValueError, LessOrEqualToZeroError)):
            Form6Dot20ShearStress(v_ed=v_ed, s=s, i=i, t=t)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"\tau_{Ed} = \frac{V_{Ed} \cdot S}{I \cdot t} = \frac{1000.000 \cdot 2000.000}{3000.000 \cdot 4.000} = 166.667 \ MPa",
            ),
            ("short", r"\tau_{Ed} = 166.667 \ MPa"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        v_ed = 1000.0
        s = 2000.0
        i = 3000.0
        t = 4.0

        # Object to test
        latex = Form6Dot20ShearStress(v_ed=v_ed, s=s, i=i, t=t).latex()

        actual = {
            "complete": latex.complete,
            "short": latex.short,
        }

        assert expected == actual[representation], f"{representation} representation failed."



"""Testing formula 5.16 of EN 1993-5:2007."""

import pytest

from blueprints.codes.eurocode.en_1993_5_2007.chapter_5_ultimate_limit_states.formula_5_16 import Form5Dot16PlasticDesignResistance
from blueprints.validations import LessOrEqualToZeroError, NegativeValueError


class TestForm5Dot16PlasticDesignResistance:
    """Validation for formula 5.16 from EN 1993-5:2007."""

    def test_evaluation(self) -> None:
        """Test the evaluation of the result."""
        # Example values
        a = 5000  # MM2
        f_y = 355  # MPA
        gamma_m0 = 1.0  # DIMENSIONLESS

        # Object to test
        formula = Form5Dot16PlasticDesignResistance(a=a, f_y=f_y, gamma_m0=gamma_m0)

        # Expected result, manually calculated
        manually_calculated_result = 1775000.0  # N

        assert formula == pytest.approx(manually_calculated_result, rel=1e-4)

    @pytest.mark.parametrize(
        ("a", "f_y", "gamma_m0"),
        [
            (-5000, 355, 1.0),  # a is negative
            (5000, -355, 1.0),  # f_y is negative
            (5000, 355, -1.0),  # gamma_m0 is negative
            (5000, 355, 0),  # gamma_m0 is zero
        ],
    )
    def test_raise_error_when_invalid_values_are_given(self, a: float, f_y: float, gamma_m0: float) -> None:
        """Test invalid values."""
        with pytest.raises((LessOrEqualToZeroError, NegativeValueError)):
            Form5Dot16PlasticDesignResistance(a=a, f_y=f_y, gamma_m0=gamma_m0)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"N_{pl,Rd} = \frac{A \cdot f_y}{\gamma_{M0}} = \frac{5000.000 \cdot 355.000}{1.000} = 1775000.000 \ N",
            ),
            ("short", r"N_{pl,Rd} = 1775000.000 \ N"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        a = 5000  # MM2
        f_y = 355  # MPA
        gamma_m0 = 1.0  # DIMENSIONLESS

        # Object to test
        latex = Form5Dot16PlasticDesignResistance(a=a, f_y=f_y, gamma_m0=gamma_m0).latex()

        actual = {
            "complete": latex.complete,
            "short": latex.short,
        }

        assert expected == actual[representation], f"{representation} representation failed."



"""Testing formula 5.8 of EN 1993-1-1:2005."""

import numpy as np
import pytest

from blueprints.codes.eurocode.en_1993_1_1_2005.chapter_5_structural_analysis.formula_5_8 import Form5Dot8CheckSlenderness
from blueprints.validations import LessOrEqualToZeroError, NegativeValueError


class TestForm5Dot8CheckSlenderness:
    """Validation for formula 5.8 from EN 1993-1-1:2005."""

    def test_evaluation(self) -> None:
        """Tests the evaluation of the result."""
        # Example values
        lambda_bar = 1.0
        a = 1000.0
        f_y = 355.0
        n_ed = 100000.0

        # Object to test
        formula = Form5Dot8CheckSlenderness(lambda_bar=lambda_bar, a=a, f_y=f_y, n_ed=n_ed)

        # Expected result, manually calculated
        expected_result = True
        expected_unity_check = lambda_bar / (0.5 * np.sqrt(a * f_y / n_ed))

        assert formula == expected_result
        assert formula.unity_check == expected_unity_check

    @pytest.mark.parametrize(
        ("lambda_bar", "a", "f_y", "n_ed"),
        [
            (1.0, 1000.0, 355.0, -100000.0),  # n_ed is negative
            (1.0, 1000.0, 355.0, 0.0),  # n_ed is zero
        ],
    )
    def test_raise_error_when_invalid_values_less_or_equal_to_zero(self, lambda_bar: float, a: float, f_y: float, n_ed: float) -> None:
        """Test invalid values."""
        with pytest.raises(LessOrEqualToZeroError):
            Form5Dot8CheckSlenderness(
                lambda_bar=lambda_bar,
                a=a,
                f_y=f_y,
                n_ed=n_ed,
            )

    @pytest.mark.parametrize(
        ("lambda_bar", "a", "f_y", "n_ed"),
        [
            (-1.0, 1000.0, 355.0, 100000.0),  # lambda_bar is negative
            (1.0, -1000.0, 355.0, 100000.0),  # a is negative
            (1.0, 1000.0, -355.0, 100000.0),  # f_y is negative
        ],
    )
    def test_raise_error_when_negative(self, lambda_bar: float, a: float, f_y: float, n_ed: float) -> None:
        """Test invalid values."""
        with pytest.raises(NegativeValueError):
            Form5Dot8CheckSlenderness(
                lambda_bar=lambda_bar,
                a=a,
                f_y=f_y,
                n_ed=n_ed,
            )

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"CHECK \to \left( \overline{\lambda} > 0.5 \sqrt{\frac{A \cdot f_{y}}{N_{Ed}}} "
                r"\right) \to \left( \overline{1.00} > 0.5 \sqrt{\frac{1000.00 \cdot 355.00}{100000.00}} \right) \to OK",
            ),
            ("short", r"CHECK \to OK"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        lambda_bar = 1.0
        a = 1000.0
        f_y = 355.0
        n_ed = 100000.0

        # Object to test
        latex = Form5Dot8CheckSlenderness(
            lambda_bar=lambda_bar,
            a=a,
            f_y=f_y,
            n_ed=n_ed,
        ).latex()

        actual = {
            "complete": latex.complete,
            "short": latex.short,
        }

        assert expected == actual[representation], f"{representation} representation failed."



"""Testing formula 5.6 of EN 1993-5:2007."""

import pytest

from blueprints.codes.eurocode.en_1993_5_2007.chapter_5_ultimate_limit_states.formula_5_6 import Form5Dot6ProjectedShearArea
from blueprints.validations import LessOrEqualToZeroError


class TestForm5Dot6ProjectedShearArea:
    """Validation for formula 5.6 from EN 1993-5:2007."""

    def test_evaluation(self) -> None:
        """Test the evaluation of the result."""
        # Example values
        h = 500  # MM
        t_f = 20  # MM
        t_w = 10  # MM

        form = Form5Dot6ProjectedShearArea(h=h, t_f=t_f, t_w=t_w)

        # Expected result, manually calculated
        expected = 4800

        assert form == pytest.approx(expected)

    @pytest.mark.parametrize(
        ("h", "t_f", "t_w"),
        [
            (500, -20, 10),  # tf is negative
            (500, 0, 10),  # tf is zero
            (500, 20, -10),  # tw is negative
            (500, 20, 0),  # tw is zero
            (0, 20, 10),  # h is zero
            (-500, 20, 10),  # h is negative
        ],
    )
    def test_raise_error_when_negative_or_zero_values_are_given(self, h: float, t_f: float, t_w: float) -> None:
        """Test a zero and negative value for parameters h, tf, and tw."""
        with pytest.raises(LessOrEqualToZeroError):
            Form5Dot6ProjectedShearArea(h=h, t_f=t_f, t_w=t_w)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"A_v = t_w \left(h - t_f \right) = 10 \cdot \left(500 - 20 \right) = 4800.00",
            ),
            ("short", r"A_v = 4800.00"),
            (
                "string",
                r"A_v = t_w \left(h - t_f \right) = 10 \cdot \left(500 - 20 \right) = 4800.00",
            ),
        ],
    )
    def test_latex_output(self, representation: str, expected: str) -> None:
        """Test the latex implementation."""
        # Example values
        h = 500  # MM
        t_f = 20  # MM
        t_w = 10  # MM

        form = Form5Dot6ProjectedShearArea(h=h, t_f=t_f, t_w=t_w).latex()

        actual = {
            "complete": form.complete,
            "short": form.short,
            "string": str(form),
        }

        assert actual[representation] == expected, f"{representation} representation failed."



"""Testing formula 6.39 of EN 1992-1-1:2004."""

import pytest

from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_6_ultimate_limit_state.formula_6_39 import Form6Dot39BetaCoefficient
from blueprints.validations import LessOrEqualToZeroError, NegativeValueError


class TestForm6Dot39BetaCoefficient:
    """Validation for formula 6.39 from EN 1992-1-1:2004."""

    def test_evaluation(self) -> None:
        """Tests the evaluation of the result."""
        # Example values
        k = 1.0
        m_ed = 100.0
        v_ed = 200.0
        u_1 = 300.0
        w_1 = 400.0

        # Object to test
        formula = Form6Dot39BetaCoefficient(k=k, m_ed=m_ed, v_ed=v_ed, u_1=u_1, w_1=w_1)

        # Expected result, manually calculated
        manually_calculated_result = 1.375  # dimensionless

        assert formula == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    @pytest.mark.parametrize(
        ("k", "m_ed", "v_ed", "u_1", "w_1"),
        [
            (-1.0, 100.0, 200.0, 300.0, 400.0),  # k is negative
            (1.0, -100.0, 200.0, 300.0, 400.0),  # m_ed is negative
            (1.0, 100.0, -200.0, 300.0, 400.0),  # v_ed is negative
            (1.0, 100.0, 200.0, -300.0, 400.0),  # u_1 is negative
            (1.0, 100.0, 200.0, 300.0, -400.0),  # w_1 is negative
            (1.0, 100.0, 0.0, 300.0, 400.0),  # v_ed is zero
            (1.0, 100.0, 200.0, 300.0, 0.0),  # w_1 is zero
        ],
    )
    def test_raise_error_when_invalid_values_are_given(self, k: float, m_ed: float, v_ed: float, u_1: float, w_1: float) -> None:
        """Test invalid values."""
        with pytest.raises((NegativeValueError, LessOrEqualToZeroError)):
            Form6Dot39BetaCoefficient(k=k, m_ed=m_ed, v_ed=v_ed, u_1=u_1, w_1=w_1)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"\beta = 1 + k \cdot \frac{M_{Ed}}{V_{Ed}} \cdot \frac{u_1}{W_1} = "
                r"1 + 1.000 \cdot \frac{100.000}{200.000} \cdot \frac{300.000}{400.000} = 1.375 \ -",
            ),
            ("short", r"\beta = 1.375 \ -"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        k = 1.0
        m_ed = 100.0
        v_ed = 200.0
        u_1 = 300.0
        w_1 = 400.0

        # Object to test
        latex = Form6Dot39BetaCoefficient(k=k, m_ed=m_ed, v_ed=v_ed, u_1=u_1, w_1=w_1).latex()

        actual = {
            "complete": latex.complete,
            "short": latex.short,
        }

        assert expected == actual[representation], f"{representation} representation failed."



"""Testing formula 7.15 of EN 1992-1-1:2004."""

import pytest

from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_7_serviceability_limit_state.formula_7_15 import Form7Dot15MaximumCrackSpacing
from blueprints.validations import GreaterThan90Error, LessOrEqualToZeroError, NegativeValueError


class TestForm7Dot15MaximumCrackSpacing:
    """Validation for formula 7.15 from EN 1992-1-1:2004."""

    def test_evaluation(self) -> None:
        """Tests the evaluation of the result."""
        # Example values
        theta = 30.0
        sr_max_y = 200.0
        sr_max_z = 300.0

        # Object to test
        formula = Form7Dot15MaximumCrackSpacing(theta=theta, sr_max_y=sr_max_y, sr_max_z=sr_max_z)

        # Expected result, manually calculated
        manually_calculated_result = 166.755778576

        assert formula == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    @pytest.mark.parametrize(
        ("theta", "sr_max_y", "sr_max_z"),
        [
            (30.0, 0.0, 300.0),  # sr_max_y is zero
            (30.0, 200.0, 0.0),  # sr_max_z is zero
            (30.0, -200.0, 300.0),  # sr_max_y is negative
            (30.0, 200.0, -300.0),  # sr_max_z is negative
            (-30.0, 200.0, 300.0),  # theta is negative
            (100.0, 200.0, 300.0),  # theta is greater than 90
            (15.0, 200.0, 300.0),  # theta is less than or equal to 15
        ],
    )
    def test_raise_error_when_invalid_values_are_given(self, theta: float, sr_max_y: float, sr_max_z: float) -> None:
        """Test invalid values."""
        with pytest.raises((LessOrEqualToZeroError, NegativeValueError, GreaterThan90Error, ValueError)):
            Form7Dot15MaximumCrackSpacing(theta=theta, sr_max_y=sr_max_y, sr_max_z=sr_max_z)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"s_{r,max} = \frac{1}{\left(\frac{\cos(\theta)}{s_{r,max,y}}\right) + \left(\frac{\sin(\theta)}{s_{r,max,z}}\right)} = "
                r"\frac{1}{\left(\frac{\cos(30.000)}{200.000}\right) + \left(\frac{\sin(30.000)}{300.000}\right)} = 166.756 \ mm",
            ),
            ("short", r"s_{r,max} = 166.756 \ mm"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        theta = 30.0
        sr_max_y = 200.0
        sr_max_z = 300.0

        # Object to test
        latex = Form7Dot15MaximumCrackSpacing(theta=theta, sr_max_y=sr_max_y, sr_max_z=sr_max_z).latex()

        actual = {
            "complete": latex.complete,
            "short": latex.short,
        }

        assert expected == actual[representation], f"{representation} representation failed."



"""Testing formula 6.52 of EN 1992-1-1:2004."""

import pytest

from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_6_ultimate_limit_state.formula_6_52 import (
    Form6Dot52PunchingShearResistance,
    Form6Dot52Sub1EffectiveYieldStrength,
)
from blueprints.validations import LessOrEqualToZeroError, NegativeValueError


class TestForm6Dot52PunchingShearResistance:
    """Validation for formula 6.52 from EN 1992-1-1:2004."""

    def test_evaluation(self) -> None:
        """Tests the evaluation of the result."""
        # Example values
        v_rd_c = 1.5  # MPa
        d = 200.0  # mm
        s_r = 150.0  # mm
        a_sw = 1000.0  # mm^2
        f_ywd_ef = 500.0  # MPa
        u_1 = 400.0  # mm
        alpha = 45.0  # degrees

        # Object to test
        formula = Form6Dot52PunchingShearResistance(v_rd_c=v_rd_c, d=d, s_r=s_r, a_sw=a_sw, f_ywd_ef=f_ywd_ef, u_1=u_1, alpha=alpha)

        # Expected result, manually calculated
        manually_calculated_result = 9.96383476483  # MPa

        assert formula == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    @pytest.mark.parametrize(
        ("v_rd_c", "d", "s_r", "a_sw", "f_ywd_ef", "u_1", "alpha"),
        [
            (-1.5, 200.0, 150.0, 1000.0, 500.0, 400.0, 45.0),  # v_rd_c is negative
            (1.5, -200.0, 150.0, 1000.0, 500.0, 400.0, 45.0),  # d is negative
            (1.5, 200.0, -150.0, 1000.0, 500.0, 400.0, 45.0),  # s_r is negative
            (1.5, 200.0, 150.0, -1000.0, 500.0, 400.0, 45.0),  # a_sw is negative
            (1.5, 200.0, 150.0, 1000.0, -500.0, 400.0, 45.0),  # f_ywd_ef is negative
            (1.5, 200.0, 150.0, 1000.0, 500.0, -400.0, 45.0),  # u_1 is negative
            (1.5, 200.0, 150.0, 1000.0, 500.0, 400.0, -45.0),  # alpha is negative
        ],
    )
    def test_raise_error_when_invalid_values_are_given(
        self, v_rd_c: float, d: float, s_r: float, a_sw: float, f_ywd_ef: float, u_1: float, alpha: float
    ) -> None:
        """Test invalid values."""
        with pytest.raises((NegativeValueError, LessOrEqualToZeroError)):
            Form6Dot52PunchingShearResistance(v_rd_c=v_rd_c, d=d, s_r=s_r, a_sw=a_sw, f_ywd_ef=f_ywd_ef, u_1=u_1, alpha=alpha)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"v_{Rd,cs} = 0.75 \cdot v_{Rd,c} + 1.5 \cdot \frac{ d}{s_r} \cdot A_{sw} \cdot "
                r"f_{ywd,ef} \cdot \frac{1}{u_{1} \cdot d} \cdot \sin(\alpha) = "
                r"0.75 \cdot 1.500 + 1.5 \cdot \frac{ 200.000}{150.000} \cdot 1000.000 \cdot 500.000 "
                r"\cdot \frac{1}{400.000 \cdot 200.000} \cdot \sin(45.000) = 9.964 \ MPa",
            ),
            ("short", r"v_{Rd,cs} = 9.964 \ MPa"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        v_rd_c = 1.5  # MPa
        d = 200.0  # mm
        s_r = 150.0  # mm
        a_sw = 1000.0  # mm^2
        f_ywd_ef = 500.0  # MPa
        u_1 = 400.0  # mm
        alpha = 45.0  # degrees

        # Object to test
        latex = Form6Dot52PunchingShearResistance(v_rd_c=v_rd_c, d=d, s_r=s_r, a_sw=a_sw, f_ywd_ef=f_ywd_ef, u_1=u_1, alpha=alpha).latex()

        actual = {
            "complete": latex.complete,
            "short": latex.short,
        }

        assert expected == actual[representation], f"{representation} representation failed."


class TestForm6Dot52Sub1EffectiveYieldStrength:
    """Validation for formula 6.52sub1 from EN 1992-1-1:2004."""

    def test_evaluation(self) -> None:
        """Tests the evaluation of the result."""
        # Example values
        d = 500.0
        f_ywd = 400.0

        # Object to test
        formula = Form6Dot52Sub1EffectiveYieldStrength(d=d, f_ywd=f_ywd)

        # Expected result, manually calculated
        manually_calculated_result = 375  # MPa

        assert formula == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    @pytest.mark.parametrize(
        ("d", "f_ywd"),
        [
            (-500.0, 400.0),  # d is negative
            (500.0, -400.0),  # f_ywd is negative
        ],
    )
    def test_raise_error_when_invalid_values_are_given(self, d: float, f_ywd: float) -> None:
        """Test invalid values."""
        with pytest.raises(NegativeValueError):
            Form6Dot52Sub1EffectiveYieldStrength(d=d, f_ywd=f_ywd)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"f_{ywd,ef} = \min\left(250 + 0.25 \cdot d, f_{ywd}\right) = \min\left(250 + 0.25 \cdot 500.000, 400.000\right) = 375.000 \ MPa",
            ),
            ("short", r"f_{ywd,ef} = 375.000 \ MPa"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        d = 500.0
        f_ywd = 400.0

        # Object to test
        latex = Form6Dot52Sub1EffectiveYieldStrength(d=d, f_ywd=f_ywd).latex()

        actual = {
            "complete": latex.complete,
            "short": latex.short,
        }

        assert expected == actual[representation], f"{representation} representation failed."



"""Formula 5.42 from EN 1992-1-1:2004: Chapter 5 - Structural Analysis."""

from blueprints.codes.eurocode.en_1992_1_1_2004 import EN_1992_1_1_2004
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula
from blueprints.type_alias import MPA
from blueprints.validations import raise_if_less_or_equal_to_zero


class Form5Dot42ConcreteCompressiveStress(Formula):
    r"""Class representing formula 5.42 for calculating the maximum concrete compressive stress."""

    label = "5.42"
    source_document = EN_1992_1_1_2004

    def __init__(
        self,
        sigma_c: MPA,
        f_ck_t: MPA,
    ) -> None:
        r"""Calculate the concrete compressive stress resulting from the prestressing force.

        EN 1992-1-1:2004 art.5.10.2.2(5) - Formula (5.42)

        Parameters
        ----------
        sigma_c : MPA
            [$\sigma_{c}$] Concrete compressive stress [MPa].
        f_ck_t : MPA
            [$f_{ck}(t)$] Characteristic compressive strength of concrete at time t [MPa].
        """
        super().__init__()
        self.sigma_c = sigma_c
        self.f_ck_t = f_ck_t

    @staticmethod
    def _evaluate(
        sigma_c: MPA,
        f_ck_t: MPA,
    ) -> bool:
        """Evaluates the formula, for more information see the __init__ method."""
        raise_if_less_or_equal_to_zero(sigma_c=sigma_c, f_ck_t=f_ck_t)

        return sigma_c <= 0.6 * f_ck_t

    def latex(self, n: int = 3) -> LatexFormula:
        """Returns LatexFormula object for formula 5.42."""
        return LatexFormula(
            return_symbol=r"CHECK",
            result="OK" if self.__bool__() else "\\text{Not OK}",
            equation=r"\sigma_{c} \leq 0.6 \cdot f_{ck}(t)",
            numeric_equation=rf"{self.sigma_c:.{n}f} \leq 0.6 \cdot {self.f_ck_t:.{n}f}",
            comparison_operator_label="\\to",
            unit="",
        )



"""Testing formula 6.7n of EN 1992-1-1:2004."""

import pytest

from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_6_ultimate_limit_state.formula_6_7n import Form6Dot7nCheckCotTheta
from blueprints.validations import GreaterThan90Error, NegativeValueError


class TestForm6Dot7nCheckCotTheta:
    """Validation for formula 6.7n from EN 1992-1-1:2004."""

    def test_evaluation(self) -> None:
        """Tests the evaluation of the result."""
        # Example values
        theta = 30.0

        # Object to test
        formula = Form6Dot7nCheckCotTheta(theta=theta)

        # Expected result, manually calculated
        expected_result = True

        assert formula == expected_result

    @pytest.mark.parametrize(
        ("theta"),
        [
            (-30.0),  # theta is negative
            (95.0),  # theta is greater than 90
        ],
    )
    def test_raise_error_when_invalid_values_are_given(self, theta: float) -> None:
        """Test invalid values."""
        with pytest.raises((NegativeValueError, GreaterThan90Error)):
            Form6Dot7nCheckCotTheta(theta=theta)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"CHECK \to 1 \leq \cot(\theta) \leq 2.5 \to 1 \leq \cot(30.000) \leq 2.5 \to OK",
            ),
            ("short", r"CHECK \to OK"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        theta = 30.0

        # Object to test
        latex = Form6Dot7nCheckCotTheta(theta=theta).latex()

        actual = {
            "complete": latex.complete,
            "short": latex.short,
        }

        assert expected == actual[representation], f"{representation} representation failed."



"""Formula 8.14 from EN 1992-1-1:2004: Chapter 8: Detailing of reinforcement and prestressing tendons."""

import numpy as np

from blueprints.codes.eurocode.en_1992_1_1_2004 import EN_1992_1_1_2004
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula
from blueprints.type_alias import DIMENSIONLESS, MM
from blueprints.validations import raise_if_negative


class Form8Dot14EquivalentDiameterBundledBars(Formula):
    """Class representing formula 8.14 for the calculation of the equivalent diameter of bundled bars, [$Ø_{n}$]."""

    label = "8.14"
    source_document = EN_1992_1_1_2004

    def __init__(
        self,
        diameter: MM,
        n_b: DIMENSIONLESS,
    ) -> None:
        r"""[$Ø_{n}$] Equivalent diameter of bundled bars [$mm$].

        EN 1992-1-1:2004 art.8.9.1(2) - Formula (8.14)

        Parameters
        ----------
        diameter : MM
            [$Ø$] Diameter of the bars [$mm$]
        n_b : DIMENSIONLESS
            [$n_{b}$] Number of bars in the bundle [$-$].

            ≤ 4 for vertical bars in compression and for bars in a lapped joint.

            ≤ 3 for all other cases.
        """
        super().__init__()
        self.diameter = diameter
        self.n_b = n_b

    @staticmethod
    def _evaluate(diameter: MM, n_b: DIMENSIONLESS) -> MM:
        """Evaluates the formula, for more information see the __init__ method."""
        raise_if_negative(
            diameter=diameter,
            n_b=n_b,
        )
        return min(diameter * np.sqrt(n_b), 55)

    def latex(self, n: int = 2) -> LatexFormula:
        """Returns LatexFormula object for formula 8.14."""
        return LatexFormula(
            return_symbol=r"Ø_n",
            result=f"{self:.{n}f}",
            equation=r"\min \left(55 \ \text{mm}, Ø \cdot \sqrt{n_b} \right)",
            numeric_equation=rf"\min \left(55 \ \text{{mm}}, {self.diameter:.{n}f} \cdot \sqrt{{{self.n_b:.{n}f}}} \right)",
            comparison_operator_label="=",
        )



"""Formula 6.53 from EN 1992-1-1:2004: Chapter 6 - Ultimate Limit State."""

from blueprints.codes.eurocode.en_1992_1_1_2004 import EN_1992_1_1_2004
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula, latex_replace_symbols
from blueprints.type_alias import DIMENSIONLESS, MM, MPA, N
from blueprints.validations import raise_if_less_or_equal_to_zero, raise_if_negative


class Form6Dot53CheckPunchingShear(Formula):
    r"""Class representing formula 6.53 for the check of punching shear resistance.

    EN 1992-1-1:2004 art.6.4.5(3) - Formula (6.53)

    Parameters
    ----------
    beta : DIMENSIONLESS
        [$\beta$] See 6.4.3 (3), (4) and (5) [$-$].
    v_ed : N
        [$V_{Ed}$] Design shear force [$N$].
    u_0 : MM
        [$u_{0}$] Perimeter of the critical section, differs for interior vs edge vs corner column [$mm$].
    d : MM
        [$d$] Mean effective depth of the slab [$mm$].
    v_rd_max : MPA
        [$v_{Rd,max}$] Maximum design shear stress [$MPa$].
    """

    label = "6.53"
    source_document = EN_1992_1_1_2004

    def __init__(
        self,
        beta: DIMENSIONLESS,
        v_ed: N,
        u_0: MM,
        d: MM,
        v_rd_max: MPA,
    ) -> None:
        super().__init__()
        self.beta = beta
        self.v_ed = v_ed
        self.u_0 = u_0
        self.d = d
        self.v_rd_max = v_rd_max

    @staticmethod
    def _evaluate(
        beta: DIMENSIONLESS,
        v_ed: N,
        u_0: MM,
        d: MM,
        v_rd_max: MPA,
    ) -> bool:
        """Evaluates the formula, for more information see the __init__ method."""
        raise_if_less_or_equal_to_zero(u_0=u_0, d=d)
        raise_if_negative(beta=beta, v_ed=v_ed, v_rd_max=v_rd_max)

        return (beta * v_ed / (u_0 * d)) <= v_rd_max

    def latex(self, n: int = 3) -> LatexFormula:
        """Returns LatexFormula object for formula 6.53."""
        _equation: str = r"\frac{\beta \cdot V_{Ed}}{u_{0} \cdot d} \leq v_{Rd,max}"
        _numeric_equation: str = latex_replace_symbols(
            _equation,
            {
                r"\beta": f"{self.beta:.{n}f}",
                r"V_{Ed}": f"{self.v_ed:.{n}f}",
                r"u_{0}": f"{self.u_0:.{n}f}",
                r" d": f" {self.d:.{n}f}",
                r"v_{Rd,max}": f"{self.v_rd_max:.{n}f}",
            },
            False,
        )
        return LatexFormula(
            return_symbol=r"CHECK",
            result="OK" if self.__bool__() else r"\text{Not OK}",
            equation=_equation,
            numeric_equation=_numeric_equation,
            comparison_operator_label=r"\to",
            unit="",
        )



"""Formula 5.6 from EN 1993-5:2007 Chapter 5 - Ultimate limit state."""

from blueprints.codes.eurocode.en_1993_5_2007 import EN_1993_5_2007
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula
from blueprints.type_alias import MM, MM2
from blueprints.validations import raise_if_less_or_equal_to_zero


class Form5Dot6ProjectedShearArea(Formula):
    """Class representing formula 5.6 for the projected shear area for each web of a U-profile or Z-profile."""

    label = "5.6"
    source_document = EN_1993_5_2007

    def __init__(
        self,
        h: MM,
        t_f: MM,
        t_w: MM,
    ) -> None:
        r"""[$A_{v}$] Calculate the projected shear area for each web of a U-profile or Z-profile in [$mm^2$].

        EN 1993-5:2007(E) art.5.2.2(5) - Formula (5.6)

        Parameters
        ----------
        h : MM
            [$h$] Overall height in [$mm$].
        t_f : MM
            [$t_{f}$] Flange thickness in [$mm$].
        t_w : MM
            [$t_{w}$] Web thickness in [$mm$].
        """
        super().__init__()
        self.h: MM = h
        self.t_f: MM = t_f
        self.t_w: MM = t_w

    @staticmethod
    def _evaluate(
        h: MM,
        t_f: MM,
        t_w: MM,
    ) -> MM2:
        """Evaluates the formula for projected shear area."""
        raise_if_less_or_equal_to_zero(h=h, t_f=t_f, t_w=t_w)
        return t_w * (h - t_f)

    def latex(self, n: int = 2) -> LatexFormula:
        """Returns LatexFormula object for formula 5.6."""
        return LatexFormula(
            return_symbol=r"A_v",
            result=f"{self:.{n}f}",
            equation=r"t_w \left(h - t_f \right)",
            numeric_equation=rf"{self.t_w} \cdot \left({self.h} - {self.t_f} \right)",
            comparison_operator_label="=",
        )
